/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.access.filters;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.model.NDRObject;

/**
 * Performs operations on an NDR object during a given stage of an NDR
 * operation.
 * <p>
 * Given a mutable NDR object and its context within an NDR operation, an
 * NDRAccess filter may chose to modify that object, terminate execution, or
 * perform some other action.
 * </p>
 * 
 * @author birkland
 * @since 1.1
 */
public interface NDRAccessFilter {
    /**
     * Invoke the filter on a particular object in a given state of NDR object
     * execution.
     * 
     * @param object
     *            The object that is involved in the NDR operation.
     * @param context
     *            Context containing the state of execution.
     */
    public void doFilter(NDRObject object, FilterContext context);

    /**
     * Initialize the filter.
     * <p>
     * Some filters require access to the repository. This method is invoked
     * exactly once, which occurs before any filter invocations occur.
     * </p>
     * 
     * @param ndr
     *            The repository in which the filter will be invoked.
     */
    public void init(NDRAccess ndr);
}
