/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.access.filters;

import org.apache.log4j.Logger;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.model.types.Type;

import static org.nsdl.repository.access.filters.FilterState.BEFORE_COMMIT;
import static org.nsdl.repository.model.content.OAIModel.Item.Components.ITEM_ID;

/**
 * Replaces a placeholder with a system assigned itemID, when requested.
 * <p>
 * If a metadata object has an itemId property with the value of "*", a real
 * itemId of the form "oai:nsdl.org:<transformed-handle>" is substituted.
 * 
 * </p>
 * 
 * @author birkland
 */
public class NSDLDefaultItemIdFilter implements NDRAccessFilter {

    private static final Logger logger = Logger
            .getLogger(NSDLDefaultItemIdFilter.class);

    public void doFilter(NDRObject object, FilterContext context) {
        if (Type.Metadata.equals(object.type())
                && BEFORE_COMMIT.equals(context.state())) {
            replaceDefaultItemID(object);
        }
    }

    public void init(NDRAccess ndr) {
        /* Nothing needed... */
    }

    private void replaceDefaultItemID(NDRObject o) {
        Property itemId = null;
        String previousValue = null;

        if ((itemId = o.getProperty(ITEM_ID)) != null) {
            previousValue = itemId.value();
        }

        if (("*".equals(previousValue) || "".equals(previousValue))) {
            String handle = o.handle();
            String oaiID = null;

            if (handle != null) {
                oaiID = "oai:nsdl.org:" + handle.replace("/", "-");
                logger.debug("Assigning default itemId " + oaiID);
            } else {
                return;
            }

            o.removeProperty(ITEM_ID, previousValue);
            o.addProperty(ITEM_ID, oaiID);

        }
    }
}
