/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.NDRObjectInfo;

public class CountMembers extends GenericAction {
    private static final Logger logger = Logger.getLogger(CountMembers.class);

    protected static final String ME = "listMembers";

    // Message properties file access is through this:
    MessageResources mrscResources = null;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
               HttpServletRequest request, HttpServletResponse response)
               throws Exception {

           // Set up error mechanism
           ActionErrors lerrErrors = new ActionErrors();

           // Cast incoming form so we can reference it
           Data lfrmData = (Data) form;

           // Set up basics
           initDataFormBean(lfrmData, request);

           // Message properties file access is through this:
           if (mrscResources == null)
               mrscResources = this.getResources(request);

           NDRAccess ndr = getNDR(request);
           Request req;
           try {
               logger.trace("Begin counting");
               req = getApiRequest(request);
               ndr = getNDR(request);

               /* Make sure an object has been specified */
               if (req.getObjectInfo() == null) {
                   throw new IllegalArgumentException(
                           "No object has been specified..");
               } 

               int count = 0;
               
               Iterator<NDRObjectInfo> members = ListMembers.list(req.getObjectInfo(), ndr).iterator();
               while (members.hasNext()) {
                   members.next();
                   count++;
               }

               lfrmData.setCount(count);

               return (mapping.findForward(SUCCESS_FORWARD));
               
           } catch (Exception e) {
               logger.error(e.getMessage());
               lfrmData
                       .setCode(mrscResources.getMessage("error.application.code"));
               lerrErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                       "error.application.api", " " + ME + ": " + e.getMessage()));
               saveMessages(request, lerrErrors);
               response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
               return (mapping.findForward(ERROR_FORWARD));
           }
       }
}
