/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.access.Results;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.api.request.Request.Commands;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.model.NDRObject.State;
import org.nsdl.repository.util.NDRObjectTemplate;

import fedora.common.Constants;

/**
 * Find an NDR object by matching properties and relationships
 * 
 * @author birkland
 * 
 */
public class Find extends GenericAction {

    private static final Logger logger = Logger.getLogger(Find.class.getName());

    private MessageResources resources;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        logger.trace("Entering execute");
        /* set up basics */
        initDataFormBean((Data) form, request);

        /* Initialize the message resources, if not initialized already */
        if (resources == null)
            resources = this.getResources(request);

        ArrayList<String> matches = new ArrayList<String>();

        try {
            /* Get the request */
            logger.trace("Getting request");
            NDRAccess ndr = getNDR(request);
            Request apiRequest = getApiRequest(request);

            Results<NDRObjectInfo> found = getMatches(apiRequest, ndr);

            try {
                for (NDRObjectInfo result : found) {
                    matches.add(result.handle());
                }
            } finally {
                found.close();
            }

            ((Data) form).setHandleList(matches);

            logger.trace("Success");
            return (mapping.findForward(SUCCESS_FORWARD));
        } catch (Exception e) {
            ActionErrors errors = new ActionErrors();
            logger.warn("Find failed", e);

            ((Data) form).setCode(resources
                    .getMessage("error.application.code"));

            errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.object.fedora", " Find: ", e.getMessage()));

            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            saveMessages(request, errors);
            return (mapping.findForward(ERROR_FORWARD));
        }
    }

    public static Results<NDRObjectInfo> getMatches(Request req, NDRAccess ndr) {

        NDRObjectTemplate template = new NDRObjectTemplate(req.getObjectInfo()
                .type());
        State s = State.Active;

        Set<Property> properties = new HashSet<Property>();
        for (Property p : req.getProperties(Commands.MATCH)) {
            if (p.predicate().getURI().equals(Constants.MODEL.STATE.uri)) {
                s = State.getInstanceFromHumanName(p.value());
            } else {
                properties.add(p);
            }
        }

        template.setState(s);

        template.addProperties(properties);
        template.addRelationships(req.getRelationships(Commands.MATCH));

        return ndr.getObjectFinder().matchObject(template);
    }
}
