/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.*;
import org.apache.struts.util.MessageResources;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.access.Results;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.request.Request;
import org.nsdl.util.uri.NormalizedURL;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.util.StaticObjectInfo;

import static org.nsdl.repository.model.content.ResourceModel.Identifiers.URL;

/*
 * Created on Dec 23, 2004 @version: .9 @author: Ellen J. Cramer
 * elly@cs.cornell.edu
 * 
 */

public class FindResource extends GenericAction {

    protected static final String ME = "findResource";

    private static final Logger logger = Logger.getLogger(FindResource.class
            .getName());

    // Message properties file access is through this:
    MessageResources mrscResources = null;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        // Cast incoming form so we can reference it
        Data lfrmData = (Data) form;

        // Set up basics
        initDataFormBean(lfrmData, request);

        // Message properties file access is through this:
        if (mrscResources == null)
            mrscResources = this.getResources(request);

        // Set up error mechanism
        ActionMessages lmsgMessage = new ActionMessages();

        Request findRequest = getApiRequest(request);
        NDRAccess ndr = getNDR(request);

        if (isCancelled(request))
            return (mapping.findForward(ERROR_FORWARD));

        // Get any & all parameters...
        String lstrURL = "";

        NDRObject found = null;

        try {
            if (findRequest.getParameter("identifier").size() > 0) {

                lstrURL = findRequest.getParameter("identifier").get(0);
                logger.trace("Finding by identifier " + lstrURL);
                NormalizedURL normalized = new NormalizedURL(lstrURL);
                normalized.normalize();
                found = ndr.getObjectFinder().findResource(URL,
                        normalized.toString());
            } else if (findRequest.getParameter("url").size() > 0) {
                lstrURL = findRequest.getParameter("url").get(0).replaceAll(
                        " ", "%20");
                logger.trace("Finding by URL " + lstrURL);
                NormalizedURL normalized = new NormalizedURL(lstrURL);
                normalized.normalize();
                found = ndr.getObjectFinder().findResource(URL,
                        normalized.toString());
            } else if (findRequest.getParameter("handle").size() > 0) {
                String handle = findRequest.getParameter("handle").get(0);
                logger.trace("Finding by handle " + handle);
                NDRObjectInfo identity = new StaticObjectInfo(handle);
                found = ndr.getObject(identity);
            } else {
                /** Basically, just to a 'find'! */
                logger.trace("Finding by inputXML parameters");
                Results<NDRObjectInfo> resultSet = Find.getMatches(findRequest,
                        ndr);
                try {
                    Iterator<NDRObjectInfo> results = resultSet.iterator();
                    if (results.hasNext()) {
                        found = ndr.getObject(results.next());

                        if (results.hasNext()) {
                            throw new RuntimeException(
                                    "More than one resource result found");
                        }
                    }
                } finally {
                    resultSet.close();
                }
            }

            if (found == null) {
                lfrmData.setCode(mrscResources
                        .getMessage("message.search.code"));
                lmsgMessage.add(ActionMessages.GLOBAL_MESSAGE,
                        new ActionMessage("message.search.resource"));
                saveMessages(request, lmsgMessage);
                return (mapping.findForward(SUCCESS_FORWARD));
            } else {
                lfrmData.setHandle(found.handle());
            }

        } catch (Exception e) {
            /** If fails, user gets an error message. */

            logger.warn("Error finding resource: " + e.getMessage());
            logger.trace("Fail trace:", e);
            lfrmData
                    .setCode(mrscResources.getMessage("error.application.code"));
            lmsgMessage.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", ME + " " + e.getMessage(),
                    " Search failed."));
            saveMessages(request, lmsgMessage);
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            return (mapping.findForward(ERROR_FORWARD));
        }

        return (mapping.findForward(SUCCESS_FORWARD));
    }
}
