/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionServlet;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.Constants;
import org.nsdl.repository.api.FedoraPlugIn;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.auth.AgentAuthInfo;
import org.nsdl.repository.api.auth.AuthGateway;
import org.nsdl.repository.api.auth.AuthorizationException;
import org.nsdl.repository.api.request.APIRequestProcessor;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.util.Configuration;
import org.nsdl.repository.util.RequestParameters;
import org.nsdl.util.DateUtility;
import org.nsdl.util.xml.NDRXMLutil;

import javax.servlet.http.HttpServletRequest;

/**
 * this is the superclass for all actions -- all fields and methods common to
 * all action classes should live here
 * 
 * @author Naomi Dushay
 */
public class GenericAction extends Action {

    public static final String ERROR_FORWARD   = "error";
    public static final String SUCCESS_FORWARD = "success";

    public static final String PARM_INPUT_XML    = "inputXML";
    public static final String DEFAULT_INPUT_XML = null;

    // no longer does the NDR related filtering process -- see NDRAccess filters for real filtering
    private static APIRequestProcessor filter;

    /**
     * Assign universal response info to the Data form bean: requestURL and
     * responseTime
     * 
     * @param form -
     *            Data form bean
     * @param request
     */
    protected static void initDataFormBean(ActionForm form,
            HttpServletRequest request) {
        Data dataFormBean = (Data) form;

        // requestURL
        String requestStr = request.getRequestURI();
        // + ( pathPart == null ? "" : pathPart)
        // + ( queryPart == null ? "" : "?" + queryPart);
        dataFormBean.setRequestURLStr(NDRXMLutil.XMLencode(requestStr));

        // responseTime
        dataFormBean.setResponseTimeStr(DateUtility
                .getCurrTimeInISO8601SecondsUTC());
    }

    protected static void checkAuth(AgentAuthInfo agent, Request request)
            throws AuthorizationException {

        if (agent.isVerified() == false) {
            throw new AuthorizationException("Agent is not authenticated");
        }

        if (agent.isAuthorizedFor(request) == false) {
            throw new AuthorizationException("Authorization error. "
                    + agent.getMessage());
        }
    }

    protected static Request getApiRequest(HttpServletRequest r) {
    	// filter initialized in setServlet in this class
        return filter.getRequest(r);
    }

    protected static NDRAccess getNDR(HttpServletRequest r) {
        // NDRAccess has the real NDR related filtering process because 
    	// it is configured to do so in config.default.repository.properties
        return (NDRAccess) r.getSession().getServletContext().getAttribute(
                Constants.NDR_ACCESS);
    }

    protected static Configuration getConfig(HttpServletRequest r) {
        return (Configuration) r.getSession().getServletContext().getAttribute(
                FedoraPlugIn.CONFIG);
    }
    
    protected static AgentAuthInfo getAuth(HttpServletRequest r) {
        return (AgentAuthInfo) r.getAttribute(AuthGateway.AUTH_INFO_KEY);
    }

    protected static String getInputXML(HttpServletRequest r) {
    	return RequestParameters.getParameter(r,PARM_INPUT_XML,DEFAULT_INPUT_XML);
    }

    public void setServlet(ActionServlet servlet) {
    	// called automatically by struts???
        super.setServlet(servlet);
        filter = (APIRequestProcessor) servlet.getServletContext()
                .getAttribute(Constants.API_REQUEST_PROCESSOR);
    }
}
