/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.content.ResourceModel;
import org.nsdl.repository.util.Configuration;

/*
 * Created on Dec 23, 2004 @version: .9 @author: Ellen J. Cramer
 * elly@cs.cornell.edu TODO
 * 
 */
public class GetContent extends GenericAction {

    protected static final String ME = "getContent";

    // Message properties file access is through this:
    MessageResources mResources = null;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        // Cast incoming form so we can reference it
        Data lfrmData = (Data) form;

        // Set up basics
        initDataFormBean(lfrmData, request);

        // Message properties file access is through this:
        MessageResources lrscResources = this.getResources(request);

        // Set up error mechanism
        ActionErrors lerrErrors = new ActionErrors();

        if (isCancelled(request))
            return (mapping.findForward(ERROR_FORWARD));

        // Make an instance of the RepositoryObject
        try {
            NDRAccess ndr = getNDR(request);
            Configuration config = getConfig(request);

            Request getContentRequest = getApiRequest(request);
            NDRObject item = ndr.getObject(getContentRequest.getObjectInfo());

            Datastream content =
                    item.getDatastream(ResourceModel.Components.CONTENT);

            String externalURL = null;
            if (content.getURL() != null) {
                String url = content.getURL().toExternalForm();
                if (!url.startsWith(config
                        .getProperty(NDRAccess.PROP_REPOSITORY_BASE))) {
                    externalURL = url;
                }
            }

            if (externalURL != null) {
                return new ActionForward(externalURL, true);
            } else {

                request.setAttribute("contentType", content.getMimeType());
                request.setAttribute("contentStream", content.getContent());
                return mapping.findForward("stream");
            }

        } catch (Exception e) {
            lfrmData
                    .setCode(lrscResources.getMessage("error.application.code"));
            lerrErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", ME, e.getMessage()));
            saveMessages(request, lerrErrors);
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            return (mapping.findForward(ERROR_FORWARD));
        }
    }
}
