/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */

package org.nsdl.repository.api.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.request.RequestUtils;
import org.nsdl.repository.api.request.RequestUtils.ApiRequestPath;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.model.ExistenceException;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.util.Configuration;
import org.nsdl.repository.util.StaticObjectInfo;

/**
 * Return datastream content to a user.
 * 
 * @author birkland
 * 
 */
public class GetDatastream extends GenericAction {
    private static final Logger logger = Logger.getLogger(GetDatastream.class);
    MessageResources resources;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        /* Initialize form data */
        Data formData = (Data) form;
        initDataFormBean(formData, request);

        /* Set up error mechanism */
        ActionErrors errors = new ActionErrors();

        /* Initialize the message resources, if not initialized already */
        if (resources == null)
            resources = this.getResources(request);

        ApiRequestPath path = RequestUtils.getPath(request);

        /* Guard against trailing slashes (which will send request here) */
        if (!path.hasPath()) {
            return new Get().execute(mapping, form, request, response);
        }

        Datastream.Name dsid = new Datastream.Name(path.getPath()[0]);

        String handle = path.getHandle();

        try {

            NDRAccess ndr = getNDR(request);
            Configuration config = getConfig(request);

            NDRObjectInfo identity = new StaticObjectInfo(handle);

            NDRObject object = ndr.getObject(identity);

            Datastream content = object.getDatastream(dsid);

            String externalURL = null;
            if (content.getURL() != null) {
                String url = content.getURL().toExternalForm();
                String thisRepository = config
                        .getProperty(NDRAccess.PROP_REPOSITORY_BASE);

                /*
                 * We will only redirect to EXTERNAL URLs, so don't set anything
                 * if we're internal
                 */
                if( !content.isCanonical()){
                    externalURL = url;
                }
            }

            if (externalURL != null) {
                return new ActionForward(externalURL, true);
            } else {
                request.setAttribute("label", content.getLabel());
                request.setAttribute("contentType", content.getMimeType());
                request.setAttribute("contentStream", content.getContent());
                return mapping.findForward("stream");
            }

        } catch (Throwable e) {
            ((Data) form).setCode(resources.getMessage("error.object.code"));
            errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", "get: " + e.getMessage()));
            saveMessages(request, errors);

            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);

            if (e instanceof ExistenceException) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Could not process Get call for "
                            + request.getPathInfo(), e);
                } else {
                    logger.info(e.getMessage());
                }
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            } else {
                logger.error("Could not process Get call for "
                        + request.getPathInfo(), e);
                response
                        .setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            }
            return (mapping.findForward(ERROR_FORWARD));
        }
    }
}
