/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */

package org.nsdl.repository.api.action;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.util.ResultsConstants;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.model.ExistenceException;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.util.Configuration;
import org.nsdl.repository.util.StreamUtil;
import org.nsdl.util.xml.NDRXMLutil;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class GetMultiple extends Get {

    private static final String RESPONSE_NS = "http://ns.nsdl.org/ndr/response_v1.00/";

    private static final Logger logger = Logger.getLogger(Get.class.getName());

    private static String repositoryBase = "";

    private MessageResources resources;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        /* Set up basics */
        initDataFormBean(form, request);

        /* Get message resources, errors */
        ActionErrors errors = new ActionErrors();

        /* Initialize the message resources, if not initialized already */
        if (resources == null)
            resources = this.getResources(request);

        Configuration config = getConfig(request);

        if (repositoryBase == "") {
            repositoryBase = config.getProperty("param.repository.baseURL", "");
        }

        try {

            NDRAccess ndr = getNDR(request);

            //-----------------------------
        	// get the list of handles
        	//-----------------------------
        	String inputXML = getInputXML(request);
        	List<String> handles = getHandles(inputXML);
        	
            //-----------------------------
            // check if a specific datastream was requested
            //-----------------------------
        	ParsedRequestPath path = new ParsedRequestPath(request);
        	Datastream.Name dsid = null;
        	if (path.hasDatastream())
        		dsid = new Datastream.Name(path.getDatastream());
            
            //-----------------------------
        	// wrap results
            //-----------------------------
        	StringBuffer xml = new StringBuffer();
            xml.append("\n    <"+ResultsConstants.ELEMENT_RESULT_DATA+">");
            if(path.hasDatastream())
            	xml.append("\n      <"+ResultsConstants.ELEMENT_DATASTREAM_LIST+">");
            else
            	xml.append("\n      <"+ResultsConstants.ELEMENT_OBJECT_LIST+">");
            
            //-----------------------------
            // get results
            //-----------------------------
            if(path.hasDatastream())
            {
                //-----------------------------
            	// use the datastream as the result
                //-----------------------------
            	String ds_name = path.getDatastream();
            	for(String handle : handles){
//logger.debug("Processing handle: "+handle+"     ds_name="+ds_name);            	
                	xml.append("\n        <"+ResultsConstants.ELEMENT_DATASTREAM);
                	xml.append(" "+ResultsConstants.ATTRIBUTE_DATASTREAM_NAME+"=\""+ds_name+"\"");
                	xml.append(" "+ResultsConstants.ATTRIBUTE_OBJECT_HANDLE+"=\""+handle+"\"");
                	xml.append(">\n");

                	try{
                    	NDRObject item = ndr.getObject(handle);
	                	Datastream content = item.getDatastream(dsid);
	                	xml.append(StreamUtil.streamToString(content.getContent(),-1));
                	} catch (Exception e) {
                		// ignore for now 
                		// TODO -- perhaps something should be done depending on the type of exception
// TODO remove the following line which is for testing only
xml.append("\nEXCEPTION: "+e.getLocalizedMessage());                		
logger.debug("EXCEPTION: "+e.getLocalizedMessage()+"    handle="+handle);
                	}
                	xml.append("\n        </"+ResultsConstants.ELEMENT_DATASTREAM+">");
                }
            } else {
                //-----------------------------
            	// use the object as the result
                //-----------------------------
            	for(String handle : handles){
//logger.debug("Processing handle: "+handle);            	
                	xml.append("\n        <"+ResultsConstants.ELEMENT_OBJECT);
                	xml.append(" "+ResultsConstants.ATTRIBUTE_OBJECT_HANDLE+"=\""+handle+"\"");
                	xml.append(">\n");

                	try{
                    	NDRObject item = ndr.getObject(handle);
                		xml.append(getAllContents(item,ndr,true));
	            	} catch (Exception e) {
	            		// ignore for now 
	            		// TODO -- perhaps something should be done depending on the type of exception
// TODO remove the following line which is for testing only
xml.append("\nEXCEPTION: "+e.getLocalizedMessage());                		
logger.debug("EXCEPTION: "+e.getLocalizedMessage()+"    handle="+handle);
	            	}
                	xml.append("\n        </"+ResultsConstants.ELEMENT_OBJECT+">");
               }
            }

            //-----------------------------
        	// close wrapper of results
            //-----------------------------
            if(path.hasDatastream())
            	xml.append("\n      </"+ResultsConstants.ELEMENT_DATASTREAM_LIST+">");
            else
            	xml.append("\n      </"+ResultsConstants.ELEMENT_OBJECT_LIST+">");
            xml.append("\n    </"+ResultsConstants.ELEMENT_RESULT_DATA+">");
            
            ((Data) form).setResponseXML(xml.toString());
        } catch (Exception e) {

            if (e instanceof ExistenceException) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Could not process Get call for "
                            + request.getPathInfo(), e);
                } else {
                    logger.info(e.getMessage());
                }
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            } else {
                logger.error("Could not process Get call for "
                        + request.getPathInfo(), e);
                response
                        .setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            }
            ((Data) form).setCode(resources.getMessage("error.object.code"));
            errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.object.fedora", "get: ", e.getMessage()));
            saveMessages(request, errors);

            return (mapping.findForward(ERROR_FORWARD));
        }

        return (mapping.findForward(SUCCESS_FORWARD));
    }

	//-------------------------------------------------------------------------------
	/*
	 * Get list of handles from response. 
	 *
	 * @param ndrResponseXML the response XML from an API call
	 * 
	 * @returns null if can't find resultData/handles xpath; otherwise, returns the returned handles
	 */
	//-------------------------------------------------------------------------------
	protected static List<String> getHandles(
		String inputXML )	
	{ 
		List<String> handles = Collections.synchronizedList(new ArrayList<String>());

        try{
			// parse result to get array from <handleList>
	        Document infoDoc = NDRXMLutil.getDocument(inputXML,false, false);
			Element rootElement = infoDoc.getDocumentElement();
	    	Element handleListElement = NDRXMLutil.getChildElement(rootElement,null,ResultsConstants.ELEMENT_HANDLES);
			NodeList childList = handleListElement.getChildNodes();
			for (int i = 0; i < childList.getLength(); i++)
			{
				Node child = childList.item(i);
				if( child.getNodeType() == Node.ELEMENT_NODE )
				{
			    	String handle = NDRXMLutil.getElementValue((Element)child);
					handles.add(handle);
				}
			}
        } catch( NullPointerException npe ) {
        	logger.debug("caught NullPointerException -- returning null");        
        	return null;
        } catch( Exception e ) {
        	logger.debug("caught Exception -- returning null");        
        	return null;
        }
		
		return handles;
	}

    public static class ParsedRequestPath {
        private final String m_apiCall;
        private final String m_datastream;

        public ParsedRequestPath(HttpServletRequest request) {
            String[] parts = request.getPathInfo().replaceFirst("^\\/", "")
                    .split("/");
            m_apiCall = parts[0];
            if (parts.length > 1) {
            	m_datastream = parts[1];
            } else {
            	m_datastream = "";
            }
        }

        public String getApiCall() {
            return m_apiCall;
        }

        public boolean hasDatastream() {
            return m_datastream.length() > 0;
        }

        public String getDatastream() {
            return m_datastream;
        }
    }

}
