/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import java.util.ArrayList;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.ExistenceException;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.content.CommonModel;
import org.nsdl.repository.model.content.MetadataModel;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.NDRObjectTemplate;

/*
 * Created on April 10, 2006 @version: .9 @author:
 */

/**
 * List members of an Aggregator or MetadataProvider
 * <p>
 * For Aggregators, finds all 'active' objects that have a <tt>memberOf</tt>
 * relationship to it. For MetadataProviders, finds all 'active' objects that
 * have a <tt>metadataProvidedBy</tt> relationship to it. Currently, there is no
 * way to specify other states (i.e. list deleted members). We should probably
 * expose that in the API as a parameter at some point..
 * </p>
 */
public class ListMembers extends GenericAction {

    private static final Logger logger = Logger.getLogger(ListMembers.class);
    protected static final String ME = "listMembers";

    // Message properties file access is through this:
    MessageResources mrscResources = null;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        // Set up error mechanism
        ActionErrors lerrErrors = new ActionErrors();

        // Cast incoming form so we can reference it
        Data lfrmData = (Data) form;

        // Set up basics
        initDataFormBean(lfrmData, request);

        // Message properties file access is through this:
        if (mrscResources == null)
            mrscResources = this.getResources(request);

        NDRAccess ndr = getNDR(request);
        Request req;
        try {
            logger.trace("Begin processing");
            req = getApiRequest(request);
            ndr = getNDR(request);

            /* Make sure an object has been specified */
            if (req.getObjectInfo() == null) {
                throw new IllegalArgumentException(
                        "No object has been specified..");
            }

            /*
             * XXX: This may be a bit dangerous for large result sets, but
             * unfortunately, struts needs it done this way.
             */
            ArrayList<String> results = new ArrayList<String>();
            for (NDRObjectInfo match : list(req.getObjectInfo(), ndr)) {
                results.add(match.handle());
            }

            lfrmData.setCount(results.size());
            lfrmData.setHandleList(results);

            return (mapping.findForward(SUCCESS_FORWARD));

        } catch (Exception e) {
            if (e instanceof ExistenceException) {
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            } else {
                response
                        .setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            }
            logger.error(e.getMessage());
            logger.debug("Fail trace: ", e);
            lfrmData
                    .setCode(mrscResources.getMessage("error.application.code"));
            lerrErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", " " + ME + ": " + e.getMessage()));
            saveMessages(request, lerrErrors);
            return (mapping.findForward(ERROR_FORWARD));
        }
    }

    protected static Iterable<NDRObjectInfo> list(NDRObjectInfo identity,
            NDRAccess ndr) {
        NDRObject container = ndr.getObject(identity);
        /* Match any type */
        NDRObject matchTemplate = new NDRObjectTemplate(null);

        /* Search over the correct relationship... */
        if (Type.Aggregator.equals(container.type())) {
            /* For aggregators, use memberOf */
            matchTemplate.addRelationship(CommonModel.Components.MEMBER_OF,
                    container);
        } else if (Type.MetadataProvider.equals(container.type())) {
            /* For MPs, use metadataprovidedBy */
            matchTemplate.addRelationship(
                    MetadataModel.Components.METADATA_PROVIDED_BY, container);
        } else {
            throw new IllegalArgumentException(container.type()
                    + " objects do not contain members");
        }

        return ndr.getObjectFinder().matchObject(matchTemplate);
    }
}
