/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.MessageResources;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.auth.AgentAuthInfo;
import org.nsdl.repository.api.auth.AuthorizationException;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.ExistenceException;

public class Purge extends GenericAction {
    private static final Logger logger = Logger.getLogger(Purge.class);

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response) {

        /* Set up basics */
        initDataFormBean(form, request);

        /* Get message resources, errors */
        MessageResources resources = getResources(request);
        ActionErrors errors = new ActionErrors();
        try {
            /* Get necessary information from the request */
            Request req = getApiRequest(request);
            AgentAuthInfo auth = getAuth(request);
            NDRAccess ndr = getNDR(request);

            /* Check permissions */
            checkAuth(auth, req);

            logger.info("Purging " + req.getObjectInfo().handle());
            ndr.purgeObject(req.getObjectInfo());

            ((Data) form).setHandle(req.getObjectInfo().handle());

        } catch (AuthorizationException e) {
            AgentAuthInfo auth = getAuth(request);
            Request req = getApiRequest(request);
            logger.info("Error authorizing purge of "
                    + req.getObjectInfo().handle() + " by "
                    + auth.agent().handle() + ": " + e.getMessage());
            logger.debug("Fail trace: \n", e);

            ((Data) form).setCode(resources.getMessage("error.security.code"));
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            errors
                    .add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                            "error.security.authorization", "Purge: "
                                    + e.getMessage()));
            saveMessages(request, errors);

            return (mapping.findForward(ERROR_FORWARD));
        } catch (Exception e) {

            if (e instanceof ExistenceException) {
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            } else {
                response
                        .setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            }
            logger.warn("Error purging object ", e);
            ((Data) form).setCode(resources.getMessage("error.object.code"));
            errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", "purge: " + e.getMessage(), e
                            .getMessage()));
            saveMessages(request, errors);

            return (mapping.findForward(ERROR_FORWARD));
        }

        logger.trace("Success");
        return mapping.findForward(SUCCESS_FORWARD);
    }
}
