/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.action.*;
import org.apache.struts.util.MessageResources;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.access.Results;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.model.content.CollectionsModel;
import org.nsdl.repository.model.content.MetadataModel;
import org.nsdl.repository.model.content.OAIModel;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.NDRObjectTemplate;

import java.util.Iterator;

public class Translate extends GenericAction {

    protected static final String ME = "Translate";

    // Message properties file access is through this:
    MessageResources mrscResources = null;

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {

        String lstrInputId;

        // Set up error mechanism
        ActionErrors lerrErrors = new ActionErrors();

        // Cast incoming form so we can reference it
        Data lfrmData = (Data) form;

        // Set up basics
        initDataFormBean(lfrmData, request);

        NDRAccess ndr = getNDR(request);

        // Message properties file access is through this:
        if (mrscResources == null)
            mrscResources = this.getResources(request);

        // Get the parameter (PID) from the request URI
        lstrInputId = lfrmData.getItemID();

        if (lstrInputId == null) {
            lfrmData
                    .setCode(mrscResources.getMessage("error.application.code"));
            lerrErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", ME + " No ItemID in request."));
            saveMessages(request, lerrErrors);
            return (mapping.findForward(ERROR_FORWARD));

        }

        // Incoming ID could be old-style oai ID:
        // oai:nsdl.org:<CollectionNA>:<UniqueId>"
        // or 'new' style:
        // oai:nsdl.org:2200-<handle identifier>
        int lintStart = lstrInputId.indexOf("oai:nsdl.org:");
        if (lintStart < 0) // Error
        {
            lfrmData
                    .setCode(mrscResources.getMessage("error.application.code"));
            lerrErrors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                    "error.application.api", ME
                            + " ItemID does not begin:  \"oai:nsdl.org:\" ."));
            saveMessages(request, lerrErrors);
            return (mapping.findForward(ERROR_FORWARD));
        }

        String lstrOldId = null;
        String lstrNewId = null;
        String lstrCollectionNA = null;
        String lstrMDpid = null;
        String lstrMDPpid = null;
        String lstrReturnId = null;

        // Strip off first part - oai:nsdl.org:
        String lstrWorkingId = lstrInputId.substring(13);

        lintStart = lstrWorkingId.indexOf(":");

        if (lintStart < 0) // New id?
        {
            lintStart = lstrWorkingId.indexOf("2200-");
            if (lintStart < 0) {
                lfrmData.setCode(mrscResources
                        .getMessage("error.application.code"));
                lerrErrors
                        .add(
                                ActionMessages.GLOBAL_MESSAGE,
                                new ActionMessage(
                                        "error.application.api",
                                        ME
                                                + " Invalid ItemId syntax.  Cannot find colon after Naming Authority."));
                saveMessages(request, lerrErrors);
                return (mapping.findForward(ERROR_FORWARD));
            } else {
                // Should be new-style
                // Get the itemId value to look for
                lstrNewId = lstrInputId;
            }

        } else {
            // Should be only old provider oai id.
            lstrOldId = lstrWorkingId.substring(lintStart + 1);
            // Clip off end - get only COllectionNA
            lstrCollectionNA = lstrWorkingId.substring(0, lintStart);
        }

        if (lstrNewId == null) {

            // Perform the ri search - old-style only
            try {

                NDRObjectTemplate providerTemplate =
                        new NDRObjectTemplate(Type.MetadataProvider);

                providerTemplate.addProperty(
                        CollectionsModel.Components.COLLECTION_NA,
                        lstrCollectionNA);

                NDRObject metadata = null;

                Results<NDRObjectInfo> providers =
                        ndr.getObjectFinder().matchObject(providerTemplate);
                try {
                    for (NDRObjectInfo provider : providers) {
                        NDRObjectTemplate metadataTemplate =
                                new NDRObjectTemplate(Type.Metadata);
                        metadataTemplate.addProperty(
                                MetadataModel.Components.UNIQUE_ID, lstrOldId);
                        metadataTemplate.addRelationship(
                                MetadataModel.Components.METADATA_PROVIDED_BY,
                                provider);

                        Iterator<NDRObjectInfo> metadataResults =
                                ndr.getObjectFinder().matchObject(
                                        metadataTemplate).iterator();

                        if (metadataResults.hasNext()) {
                            metadata = ndr.getObject(metadataResults.next());
                        }

                    }
                } finally {
                    providers.close();
                }

                if (metadata == null) {
                    lfrmData.setCode(mrscResources
                            .getMessage("error.application.code"));
                    lerrErrors
                            .add(
                                    ActionMessages.GLOBAL_MESSAGE,
                                    new ActionMessage(
                                            "error.application.api",
                                            ME
                                                    + " Could not find metadata object with uniqueId ("
                                                    + lstrOldId
                                                    + ") and metadata provider id ("
                                                    + lstrMDPpid + ")."));
                    saveMessages(request, lerrErrors);
                    return (mapping.findForward(ERROR_FORWARD));
                }

                Property itemId =
                        metadata.getProperty(OAIModel.Item.Components.ITEM_ID);

                if (itemId == null) {
                    lfrmData.setCode(mrscResources
                            .getMessage("error.application.code"));
                    lerrErrors
                            .add(
                                    ActionMessages.GLOBAL_MESSAGE,
                                    new ActionMessage(
                                            "error.application.api",
                                            ME
                                                    + " Could not find the OAI id in the metadata object ("
                                                    + lstrMDpid + ")."));
                    saveMessages(request, lerrErrors);
                    return (mapping.findForward(ERROR_FORWARD));
                }

                lstrReturnId = itemId.value();

            } catch (Exception e) {
                /** If fails, user gets an error message. */
                lfrmData.setCode(mrscResources
                        .getMessage("error.application.code"));
                lerrErrors.add(ActionMessages.GLOBAL_MESSAGE,
                        new ActionMessage("error.application.api", ME,
                                "Resource Index search failed."));
                saveMessages(request, lerrErrors);
                return (mapping.findForward(ERROR_FORWARD));
            }
        } else {

            try {
                NDRObjectTemplate metadataTemplate =
                        new NDRObjectTemplate(Type.Metadata);
                metadataTemplate.addProperty(OAIModel.Item.Components.ITEM_ID,
                        lstrInputId);

                Iterator<NDRObjectInfo> foundMetadata =
                        ndr.getObjectFinder().matchObject(metadataTemplate)
                                .iterator();

                if (!foundMetadata.hasNext()) {
                    lfrmData.setCode(mrscResources
                            .getMessage("error.application.code"));
                    lerrErrors
                            .add(
                                    ActionMessages.GLOBAL_MESSAGE,
                                    new ActionMessage(
                                            "error.application.api",
                                            ME
                                                    + " Could not find an object with the itemId: "
                                                    + lstrNewId + ")."));
                    saveMessages(request, lerrErrors);
                    return (mapping.findForward(ERROR_FORWARD));
                }

                NDRObject metadata = ndr.getObject(foundMetadata.next());

                NDRObject provider =
                        ndr.getObject(metadata.getRelationship(
                                MetadataModel.Components.METADATA_PROVIDED_BY)
                                .related());

                Property uniqueId =
                        metadata
                                .getProperty(MetadataModel.Components.UNIQUE_ID);

                String lstrUniqueId = null;

                if (uniqueId != null) {
                    lstrUniqueId = uniqueId.value();
                }

                Property collectionNA =
                        provider
                                .getProperty(CollectionsModel.Components.COLLECTION_NA);
                if (collectionNA != null) {
                    lstrCollectionNA = collectionNA.value();
                }

                lstrReturnId =
                        "oai:nsdl.org:" + lstrCollectionNA + ":" + lstrUniqueId;
            } catch (Exception e) {
                /** If fails, user gets an error message. */
                lfrmData.setCode(mrscResources
                        .getMessage("error.application.code"));
                lerrErrors.add(ActionMessages.GLOBAL_MESSAGE,
                        new ActionMessage("error.application.api", ME,
                                "Resource Index search failed."));
                saveMessages(request, lerrErrors);
                return (mapping.findForward(ERROR_FORWARD));
            }
        }

        if (isCancelled(request))
            return (mapping.findForward(ERROR_FORWARD));

        lfrmData.setItemID(lstrReturnId);

        return (mapping.findForward(SUCCESS_FORWARD));

    }
}
