/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.actionForm;

/*
 * Created on Apr 14, 2005 8:35:01 AM by Tim
 *
 * Data.java
 * @version $Id: 
 */

/**
 * This class is a lightweight class for transferring data between the UI and
 * action classes within the STRUTS enviropnment.
 */

import org.apache.struts.validator.*;
import java.util.*;

public class Data extends ValidatorForm {

    public static final long serialVersionUID = 1;

    /** Security info */
    protected String mstrSignature;

    protected String mstrExpires;

    protected String mstrKeyId;

    /** Input parameters */
    protected String mstrInputXML;

    protected String mstrPID;

    protected String mstrView;

    // used in the error.jsp as the error code.
    protected String mstrCode;

    // Value to tell if the object with our PID exists
    protected boolean mblnExists;

    // MetadataFormat parameter
    protected String mstrMetadataFormat;

    // XSL Object for OAI transforms
    protected String mstrXSLPID;

    /**
     * The timestamp of this response, as a UTC string used by all response xml
     */
    private String mstrResponseTimeStr;

    /** The requestURL used by all response xml */
    private String mstrRequestURLStr;

    /** Values for info.jsp from non-describe calls. */
    protected ArrayList<String> marrPIDList;

    protected ArrayList<String> marrHandleList;

    /**
     * List containing all metadata items.
     */
    protected ArrayList<String> marrMetadata;

    protected int mintCount;

    protected String mstrMetadataFormats;

    protected String mstrURL;

    /** Values for the describe.jsp and describeHTML.jsp */
    protected String mstrCreatedDate;

    protected String mstrLastModifiedDate;

    protected String mstrState;

    protected String mstrLabel;

    protected String mstrDC;

    protected String mstrRel;

    protected String mstrTarget;

    protected ArrayList<Map<String, String>> marrRelationships;

    protected String mstrDSIDs;

    protected String mstrContentDSIDList;

    protected String mstrFormatDSIDList;

    protected String mstrServiceDescriptionDSIDList;

    protected String mstrDataSourceDSIDList;

    protected String mstrOaiInfoDSIDList;

    protected ArrayList<Map<String, String>> marrRelatedObjects;

    protected String mstrHandle;

    protected String mstrHandleURL;

    protected String mstrString;

    protected String responseXML;

    protected String mstrItemID;

    // NOTE - for testing ONLY!!! remove these.
    protected String mstrProperties;

    protected String mstrRelationships;

    protected String mstrObjectxml;

    public Data() {
    }

    /** Getters and Setters */

    /**
     * Gets the response time of this response
     * 
     * @return The response time, as a UTC string in RFC8601 Z format
     */
    public String getResponseTimeStr() {
        return mstrResponseTimeStr;
    }

    /** Sets the response time of this response */
    public void setResponseTimeStr(String responseTimeStr) {
        mstrResponseTimeStr = responseTimeStr;
    }

    /**
     * Gets the request URL, in string form
     * 
     * @return The request URL, as a string
     */
    public String getRequestURLStr() {
        // TODO: need to worry about XML, URL or other char encodings?
        return mstrRequestURLStr;
    }

    /** Sets the request URL for this response */
    public void setRequestURLStr(String requestURLStr) {
        mstrRequestURLStr = requestURLStr;
    }

    /** InputXML from post */
    public String getInputXML() {
        return mstrInputXML;
    }

    /** InputXML from post */
    public void setInputXML(String lstrInputXML) {
        mstrInputXML = lstrInputXML;
    }

    /** "View" from post */
    public String getView() {
        return mstrView;
    }

    /** "View" from post */
    public void setView(String lstrView) {
        mstrView = lstrView;
    }

    /** "Code" from post */
    public String getCode() {
        return mstrCode;
    }

    /** "Code" from post */
    public void setCode(String lstrCode) {
        mstrCode = lstrCode;
    }

    /** PID from url or from action classes */
    public String getPID() {
        return mstrPID;
    }

    /** PID from url or from action classes */
    public void setPID(String lstrPID) {
        mstrPID = lstrPID;
    }

    /** info.jsp list of PID results */
    public ArrayList<String> getPIDList() {
        return marrPIDList;
    }

    /** PID from url or from action classes */
    public void setPIDList(ArrayList<String> larrPIDList) {
        marrPIDList = larrPIDList;
    }

    /** info.jsp count from activateMetadata or deactivateMetadata */
    public int getCount() {
        return mintCount;
    }

    /** info.jsp count from activateMetadata or deactivateMetadata */
    public void setCount(int lintCount) {
        mintCount = lintCount;
    }

    /** info.jsp fist of formats for listMetadataFormats */
    public String getMetadataFormats() {
        return mstrMetadataFormats;
    }

    /** info.jsp formats for listMetadataFormats */
    public void setMetadataFormats(String lstrMetadataFormats) {
        mstrMetadataFormats = lstrMetadataFormats;
    }

    /** info.jsp list of metadata items */
    public ArrayList<String> getMetadata() {
        return marrMetadata;
    }

    /** info.jsp list of metadata items */
    public void setMetadata(String lstrMetadata) {
        if (marrMetadata == null) {
            marrMetadata = new ArrayList<String>();
        }

        marrMetadata.add(lstrMetadata);
    }

    /** info.jsp URL for serviceDescription or other datastream ?? */
    public String getURL() {
        return mstrURL;
    }

    /** info.jsp URL for serviceDescription or other datastream ?? */
    public void setURL(String lstrURL) {
        mstrURL = lstrURL;
    }

    /** info.jsp Handle for Resource */
    public String getHandle() {
        return mstrHandle;
    }

    /** info.jsp Handle for Resource. */
    public void setHandle(String lstrHandle) {
        mstrHandle = lstrHandle;
    }

    /** info.jsp Handle for Resource */
    public String getHandleURL() {
        return mstrHandleURL;
    }

    /** info.jsp Handle for Resource. */
    public void setHandleURL(String lstrHandleURL) {
        mstrHandleURL = lstrHandleURL;
    }

    /** info.jsp list of PID results */
    public ArrayList<String> gethandleList() {
        return marrHandleList;
    }

    /** PID from url or from action classes */
    public void setHandleList(ArrayList<String> larrHandleList) {
        marrHandleList = larrHandleList;
    }

    /** describe.jsp property value */
    public String getCreatedDate() {
        return mstrCreatedDate;
    }

    /** describe.jsp property value */
    public void setCreatedDate(String lstrCreatedDate) {
        mstrCreatedDate = lstrCreatedDate;
    }

    /** describe.jsp property value */
    public String getLastModifiedDate() {
        return mstrLastModifiedDate;
    }

    /** describe.jsp property value */
    public void setLastModifiedDate(String lstrLastModifiedDate) {
        mstrLastModifiedDate = lstrLastModifiedDate;
    }

    /** describe.jsp property value */
    public String getState() {
        return mstrState;
    }

    /** describe.jsp property value */
    public void setState(String lstrState) {
        mstrState = lstrState;
    }

    /** describe.jsp property value */
    public String getLabel() {
        return mstrLabel;
    }

    /** describe.jsp property value */
    public void setLabel(String lstrLabel) {
        mstrLabel = lstrLabel;
    }

    /** describe.jsp relationship */
    public String getRel() {
        return mstrRel;
    }

    public void setRel(String lstrVal) {
        mstrRel = lstrVal;
    }

    /** describe.jsp relationship target */
    public String getTarget() {
        return mstrTarget;
    }

    public void setTarget(String lstrVal) {
        mstrTarget = lstrVal;
    }

    /** datastream list */
    public String getDSIDs() {
        return mstrDSIDs;
    }

    /** sets a list of all the datastreams. */
    public void setDSIDs(String lstrStreamIDList) {

        mstrDSIDs = lstrStreamIDList;
    }

    /** Get a list of content datastreams. */
    public String getContentDSIDList() {
        return mstrContentDSIDList;
    }

    /** sets a list of content the datastreams. */
    public void setContentDSIDList(String lstrSpecificIDList) {
        mstrContentDSIDList = lstrSpecificIDList;
    }

    /** Get a list of format datastreams. */
    public String getFormatDSIDList() {
        return mstrFormatDSIDList;
    }

    /** sets a list of format the datastreams. */
    public void setFormatDSIDList(String lstrSpecificIDList) {
        mstrFormatDSIDList = lstrSpecificIDList;
    }

    /** Get a list of ServiceDescription datastreams. */
    public String getServiceDescriptionDSIDList() {
        return mstrServiceDescriptionDSIDList;
    }

    /** sets a list of ServiceDescription the datastreams. */
    public void setServiceDescriptionDSIDList(String lstrSpecificIDList) {
        mstrServiceDescriptionDSIDList = lstrSpecificIDList;
    }

    /** Get a list of dataSource datastreams. */
    public String getDataSourceDSIDList() {
        return mstrDataSourceDSIDList;
    }

    /** sets a list of dataSource the datastreams. */
    public void setDataSourceDSIDList(String lstrSpecificIDList) {
        mstrDataSourceDSIDList = lstrSpecificIDList;
    }

    /** Get a list of oaiInfo datastreams. */
    public String getOaiInfoDSIDList() {
        return mstrOaiInfoDSIDList;
    }

    /** sets a list of oaiInfo the datastreams. */
    public void setOaiInfoDSIDList(String lstrSpecificIDList) {
        mstrOaiInfoDSIDList = lstrSpecificIDList;
    }

    public void setRelationships(
            ArrayList<Map<String, String>> larrRelationships) {
        marrRelationships = larrRelationships;
    }

    public ArrayList<Map<String, String>> getRelationships() {
        return marrRelationships;
    }

    public void setRelatedObjects(
            ArrayList<Map<String, String>> larrRelatedObjects) {
        marrRelatedObjects = larrRelatedObjects;
    }

    public ArrayList<Map<String, String>> getRelatedObjects() {
        return marrRelatedObjects;
    }

    public void setExists(boolean lblnValue) {
        mblnExists = lblnValue;
    }

    public boolean getExists() {
        return mblnExists;
    }

    // NOTE FOR TESTING ONLY!!! REMOVE THESE
    public String getProperties() {
        return mstrProperties;
    }

    public void setProperties(String lstrVal) {
        mstrProperties = lstrVal;
    }

    public String getRelationshipsStream() {
        return mstrRelationships;
    }

    public void setRelationshipsStream(String lstrVal) {
        mstrRelationships = lstrVal;
    }

    public String getString() {
        return mstrString;
    }

    public void setString(String lstrVal) {
        mstrString = lstrVal;
    }

    public String getObjectxml() {
        return mstrObjectxml;
    }

    public void setObjectxml(String lstrVal) {
        mstrObjectxml = lstrVal;
    }

    public void setFormat(String lstrVal) {
        mstrMetadataFormat = lstrVal;
    }

    public String getFormat() {
        return mstrMetadataFormat;
    }

    public void setXsl(String lstrVal) {
        mstrXSLPID = lstrVal;
    }

    public String getXsl() {
        return mstrXSLPID;
    }

    public void setItemID(String lstrVal) {
        mstrItemID = lstrVal;
    }

    public String getItemID() {
        return mstrItemID;
    }

    /** "Signature" from post */
    public String getSignature() {
        return mstrSignature;
    }

    /** "Signature" from post */
    public void setSignature(String lstrVal) {
        mstrSignature = lstrVal;
    }

    /** "Expires" from post */
    public String getExpires() {
        return mstrExpires;
    }

    /** "Expires" from post */
    public void setExpires(String lstrVal) {
        mstrExpires = lstrVal;
    }

    /** "KeyId" from post */
    public String getKeyId() {
        return mstrKeyId;
    }

    /** "Signature" from post */
    public void setKeyId(String lstrVal) {
        mstrKeyId = lstrVal;
    }

    public void setResponseXML(String response) {
        this.responseXML = response;
    }

    public String getResponseXML() {
        return this.responseXML;
    }
}
