/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth;

import java.security.KeyManagementException;
import java.security.PublicKey;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * Caches keys to eliminate expensive lookups for each request
 * 
 * @author birkland
 * 
 */
public class KeyManagerCache implements KeyManager {
    private final KeyManager manager;
    private final ConcurrentMap<String, PublicKey> pubKeyCache = new ConcurrentHashMap<String, PublicKey>();

    public KeyManagerCache(KeyManager managerDelegate) {
        this.manager = managerDelegate;
    }

    public PublicKey forcePubKey(String agent) throws KeyManagementException {
        pubKeyCache.put(agent, manager.getPubKey(agent));
        return pubKeyCache.get(agent);
    }

    public PublicKey getPubKey(String agent) throws KeyManagementException {
        if (!pubKeyCache.containsKey(agent)) {
            pubKeyCache.putIfAbsent(agent, manager.getPubKey(agent));
        }
        return pubKeyCache.get(agent);
    }
}
