/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth.rules;

import java.util.Set;

import org.nsdl.repository.api.auth.AgentAuthInfo;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.Component;
import org.nsdl.repository.model.Component.Pattern;

public interface ComponentRule {
	
	/** Returns all component patterns for which this rule applies.
	 * 
	 * @return Set of matching component patterns.
	 */
	public Set<Pattern> matchingPatterns();

	/** Determines if the given component is compliant with this particular rule
	 * when acted upon by the given authorized agent.
	 * 
	 * <p>
	 * Returns true if the component satisfies the rule's requiements or if the
	 * rule is not applicable to the given component (i.e.
	 * the component's pattern does not match any this component's declared
	 * patterns).
	 * </p>
	 * 
	 * <p\>
	 * Note: The evaluation the component's compliance with the rule is done in
	 * the context of an agent.  If the agent is not authorized to create, delete, or
	 * modify the given component, the component will be found to violate the
	 * rule.
	 * </p>
	 * 
	 * @param component The component to check rule compliance on
	 * @param requestContext The request from which the component originated
	 * @param agentInfo The agent that provides context for evaluating the rule
	 * @return false if this rule is applicable to the given component, and the
	 * component violates this rule
	 */
	public boolean checkCompliance(Component component, 
			Request requestContext, AgentAuthInfo agentInfo);
}
