/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth.rules;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.util.Configuration;
import org.nsdl.repository.util.Configuration.Listener;

/**
 * Manages the rules used for verifying API requests
 * 
 * Maintains a single pool of rules and provides methods for adding and
 * retrieving sets of rules that that may apply to particular requests.
 * 
 * Currently, this class is implemented as a singleton with the intention that
 * one common pool of rules would be used by the entire application
 * 
 * @author birkland
 * 
 */
public final class RuleManager implements Listener {

    private Configuration m_config = new Configuration();

    /**
     * Returns a RelationshipRuleSet that is applicable to the given request
     * 
     * @param request
     *            NDR API request
     * @return RelationshipRuleSet containing rules that apply to the given
     *         request
     */
    public RuleSet getRules(Request request, NDRAccess ndr) {
        return new DefaultRuleSet(ndr, m_config);
    }

    /**
     * Adds relationships to the pool of rules this manager may use
     * 
     * @param ruleSet
     *            Rules to add to the RuleManager
     */
    public static void addRuleSet(RuleSet ruleSet) {
        /* TODO: logic to add and use new rule sets */
    }

    public void updateConfig(Configuration config) {
        m_config = config;
    }
}
