/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth.rules;


import org.nsdl.repository.api.auth.AgentAuthInfo;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.Component;

/** Contains a set of rules, and methods for matching and applying them
 * 
 * @author birkland
 *
 */
public interface RuleSet {
	/** Determines if this rule set applies to a given request component 
	 * 
	 * @param component The component to test against this set
	 * @return <b>true</b> if this set contains at least one rule that is relevant for
	 * 	the given component
	 */
	public boolean matches(Component component);
	
	
	/** Determines if the given component is allowed by all the rules in the set
	 * 
	 * 
	 * @param component The component to test against this set
	 * @param requestContext the Request from with the component originated
	 * @param agentAuth Agent authorization info that may be used in determining rule compliance
	 * @return <b>True</b> if the component complies with every applicable rule
	 */
	public boolean allows(Component component, 
			Request requestContext, AgentAuthInfo agentAuth);
}
