/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.request;

import java.util.Set;

import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.model.Relationship;
import org.nsdl.repository.util.NDRObjectTemplate;

import static org.nsdl.repository.api.request.Request.Commands.MATCH;

/**
 * Formulates a request for matching objects
 * <p>
 * Primarily intended for internal use, builds a minimal 'match' request for
 * finding objects based upon given properties and relationships that are
 * desired to be matched
 * </p>
 * 
 * @author birkland
 * 
 */
public class MatchRequest extends MutableRequestWrapper {

    public MatchRequest(org.nsdl.repository.model.types.Type<? extends NDRObject> t) {
        NDRObjectTemplate obj = new NDRObjectTemplate(t);
        setObjectInfo(obj);
        setRequestType(Request.Type.Read);
    }

    public void addProperty(Property p) {
        putProperty(MATCH, p);
    }

    public void addProperties(Set<Property> properties) {
        for (Property p : properties) {
            addProperty(p);
        }
    }

    public void addRelationship(Relationship r) {
        putRelationship(MATCH, r);
    }

    public void addRelationships(Set<Relationship> relationships) {
        for (Relationship r : relationships) {
            addRelationship(r);
        }
    }

}
