/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.handle;

import org.nsdl.repository.access.filters.HandleManagementFilter.HandleException;
import org.nsdl.repository.access.filters.HandleManagementFilter.HandleFactory;
import org.nsdl.repository.impl.fedora.FedoraAccess;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.util.Configuration;
import org.nsdl.repository.util.ConfigurationFactory;
import org.nsdl.repository.util.Configuration.Listener;

public class FedoraPIDHandleFactory implements HandleFactory, Listener {

    private FedoraAccess m_fedora;
    private String m_pidNamespace;
    private static final String PROP_FEDORA_ACCESS = "repository.handle.FedoraPIDHandleFactory.FedoraAccess";
    private static final String PROP_PID_NAMESPACE = "repository.handle.FedoraPIDHandleFactory.namespace";

    public void deleteHandle(String handle) throws HandleException {
        /* Nothing to do - not a persistent store */
    }

    public String getHandle(NDRObject o) throws HandleException {
        try {
            return m_fedora.getAPI().getNextPID("1", m_pidNamespace)[0];
        } catch (Exception e) {
            throw new HandleException("Could not get a new handle", e);
        }
    }

    public void updateConfig(Configuration config) {
        m_fedora = ConfigurationFactory.getInstance(FedoraAccess.class)
                .buildFromProperty(PROP_FEDORA_ACCESS, config);
        m_pidNamespace = config.getProperty(PROP_PID_NAMESPACE);
    }

}
