/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.handle;

/**
 * Interface for HandleGenerators.
 * The implementation is responsible for maintaining or 
 * delegating any state to ensure generated handles are unique.
 * 
 * @author Edwin Shin
 * @version $Id $
 */
public interface HandleGenerator {
    /**
     * Generate a handle for the given naming authority.
     * 
     * @param namingAuthority
     * @return a handle
     */
    public String generateHandle(String namingAuthority);
    
    /**
     * Never generate the following handle.
     * (e.g., for a handle not created by HandleGenerator)
     * @param handle
     */
    public void neverGenerate(String handle);
}
