/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.handle;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import net.handle.hdllib.AbstractResponse;
import net.handle.hdllib.HandleException;
import net.handle.hdllib.ListHandlesResponse;
import net.handle.hdllib.ResponseMessageCallback;
import net.handle.hdllib.Util;

/**
 * @author Edwin Shin
 */
public class HandleIterator implements Iterator<String>,
        ResponseMessageCallback {
    List<String> handles;
    Iterator<String> hit;

    public HandleIterator() {
        handles = new ArrayList<String>();
        hit = handles.iterator();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * net.handle.hdllib.ResponseMessageCallback#handleResponse(net.handle.hdllib
     * .AbstractResponse)
     */
    public void handleResponse(AbstractResponse response)
            throws HandleException {
        if (response instanceof ListHandlesResponse) {
            ListHandlesResponse lhResp = (ListHandlesResponse) response;

            byte[][] hdls = lhResp.handles;
            for (int i = 0; i < hdls.length; i++) {
                handles.add(Util.decodeString(hdls[i]));
            }

            if (!response.continuous) {
                hit = handles.iterator();
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Iterator#remove()
     */
    public void remove() {
        throw new UnsupportedOperationException(
                "HandleIterator does not support remove()");
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.util.Iterator#hasNext()
     */
    public boolean hasNext() {
        return hit.hasNext();
    }

    /**
     * Returns the String value of the next handle
     */
    public String next() {
        return hit.next();
    }

}
