/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */

package org.nsdl.repository.impl.fedora;

import org.nsdl.repository.error.RepositoryException;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.impl.MutableObjectBase;
import org.nsdl.repository.model.types.Type;

import fedora.common.PID;

abstract class FedoraMutableObjectBase extends MutableObjectBase implements
        FedoraNDRObject {
    private PID pid;

    protected void initNew(Type<? extends NDRObject> type, String handle,
            PID pid) {
        initNew(type, handle);
        this.pid = pid;
    }

    protected void init(FedoraNDRObject initial) {
        this.pid = initial.pid();
        super.init(initial);
    }

    public PID pid() {
        return pid;
    }

    public void setPID(PID pid) {
        if (this.pid == null) {
            this.pid = pid;
        } else if (!this.pid.equals(pid)) {
            throw new RepositoryException(
                    "Cannnot re-assign a pid!  Existing is "
                            + this.pid.toString() + ", was asked to change to "
                            + pid.toString());
        }
    }

    public Datastream getDatastream(String id) {

        for (Datastream d : getDatastreams()) {
            if (d.getName().getID().equals(id))
                return d;
        }
        return null;
    }
}
