/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.impl.fedora;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URL;

import org.apache.log4j.Logger;
import org.nsdl.repository.access.AccessException;
import org.nsdl.repository.util.StreamUtil;

public class ManagedDatastream implements FedoraDatastream {
    private static final Logger logger = Logger
            .getLogger(ManagedDatastream.class);
    private final ControlGroup CONTROL_GROUP = ControlGroup.Managed;
    private final Name m_dsID;
    private byte[] m_content_bytes;
    private String m_label;
    private String m_mime;

    public ManagedDatastream(Name id, InputStream content, String label,
            String mime) {
        m_dsID = id;
        m_mime = mime;
        m_label = label;
        setContent(content);
    }

    /** {@inheritDoc} */
    public ControlGroup controlGroup() {
        return CONTROL_GROUP;
    }

    public String[] getAltIDs() {
        return new String[0];
    }

    public InputStream getContent() throws AccessException {
        return new ByteArrayInputStream(m_content_bytes);
    }

    public InputStream getContent(String version) throws AccessException {
        return new ByteArrayInputStream(m_content_bytes);
    }

    public void setContent(InputStream input) {
        /*
         * XXX: We're putting this into an in-memory byte array, so we hope it's
         * not big.
         */

        this.m_content_bytes = StreamUtil.streamToByteArray(input);
    }

    public String getFormatURI() {
        return null;
    }

    public Name getName() {
        return m_dsID;
    }

    public void setLabel(String label) {
        m_label = label;
    }

    public String getLabel() {
        return m_label;
    }

    public String getMimeType() {
        return m_mime;
    }

    public void setMimeType(String mimetype) {
        m_mime = mimetype;
    }

    /**
     * Get the URL for this datastream.
     * <p>
     * ManagedDatastream does not necessarily exist in a repository yet, so does
     * not have a URL;
     * </p>
     * 
     * @return null
     */
    public URL getURL() {
        logger
                .warn("BOGUS: XMLDatastream does not know its url.  Using fake value");
        return null;
    }

    public void setURL(URL url) {
        throw new UnsupportedOperationException(
                "Cannot set the URL for inline XML datastreams");
    }

    public long getSize() {
        return m_content_bytes.length;
    }

    public State getState() {
        return State.Active;
    }

    public boolean isVersionable() {
        return true;
    }

    public boolean isCanonical() {
        return true;
    }
}
