/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.impl.fedora.ri;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.nsdl.mptstore.query.QueryException;
import org.nsdl.mptstore.query.provider.SQLProvider;

public class PreparedQuery implements SQLProvider {
    public static final String PLACEHOLDER = "<info:mptstore/placeholder>";

    private static final Logger logger = Logger
            .getLogger(PreparedSQLUnionQueryResults.class);
    
    private final List<String> queries = new ArrayList<String>();
    private final List<String> targets;
    
    public PreparedQuery(SQLProvider provider, final String varName) throws QueryException {
        
        for (String rawSQL : provider.getSQL()) {
            logger.trace("Given raw SQL " + rawSQL);
            String sql = rawSQL.replaceAll("'" + varName + "'", "?");
            logger.trace("Converted to " + sql);

            queries.add(sql);
        }
        
        this.targets = provider.getTargets();
    }
    
    public List<String> getSQL() {
        return new ArrayList<String>(queries);
    }
    
    public List<String> getTargets() {
        return new ArrayList<String>(targets);
    }
}
