/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.impl.fedora.ri;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import java.util.Iterator;

import org.nsdl.repository.api.Constants;

/**
 * 
 * 
 * STOLEN! from Aaron's NDRFedoraPubKeyManager.java. ..." Need to put into place
 * a formal SPO mechanism for the repository API. For the time being, hiding
 * this implementation as an inner class"
 * 
 * @author Aaron, Tim
 * 
 */
public abstract class SPOUtils
{
	
		
	/**
	 * Convenience method to return an ArrayList of 
	 * entries matching the (single) null value of subject, predicate,
	 * or object in the query.
	 * Queries for any two parameters of a spo query.  
	 * @param fedoraURL
	 * @param Subject
	 * @param Predicate
	 * @param Object
	 * @return List of values for whatever param was null on input
	 */
	public static ArrayList<String> getList(URL fedoraURL, String Subject,
			String Predicate, String Object) 
	{
				
		ArrayList<String> larrResult = new ArrayList<String>();
		int lintReturnWhich = 0;

		try
		{
		Set<Triple> results = SPOUtils.doSearch(fedoraURL, Subject,
				Predicate, Object);

		if (Subject == null)
			lintReturnWhich = 1;
		if (Predicate == null)
			lintReturnWhich = 2;
		if (Object == null)
			lintReturnWhich = 3;

		if (results.size() < 1)
			return larrResult;		
		
		Iterator<Triple> liter = results.iterator();
		
		String lstrFound = "";

		while (liter.hasNext())
		{
			switch (lintReturnWhich)
			{
			case 1:
				lstrFound = (liter.next()).subject; 
				break;
			case 2:
				lstrFound = (liter.next()).predicate;
				break;
			case 3:
				lstrFound = (liter.next()).object;
				break;
			}

			larrResult.add(lstrFound);
		}

		} catch (SPOException e)
		{
			//  ??
			throw new RuntimeException(e);
		}
		
		return larrResult;
		
		
	}

	public static Set<Triple> doSearch(URL lurlFedora, String lstrSubject,
			String lstrPredicate, String lstrObject) throws SPOException
	{
		String queryBase = "type=triples&lang=spo&format=N-Triples&stream=on&query=";

		lstrObject = checkObject(lstrObject);

		HashSet<Triple> resultSet = new HashSet<Triple>();
		String queryString = (lstrSubject == null ? "*" : lstrSubject) + " "
				+ (lstrPredicate == null ? "*" : lstrPredicate) + " "
				+ (lstrObject == null ? "*" : lstrObject);

		URL searchURL;
		BufferedReader content;
		try
		{
			searchURL = new URL(lurlFedora.getProtocol(), lurlFedora.getHost(),
					lurlFedora.getPort(), Constants.RESOURCE_INDEX_URL + queryBase
							+ URLEncoder.encode(queryString, "US-ASCII"));

			content = new BufferedReader(
					new InputStreamReader(searchURL.openStream()));

			for (String line = content.readLine(); line != null; line = content
					.readLine())
			{
				resultSet.add(readNTriple(line));
			}

			content.close();

		} catch (MalformedURLException e)
		{
			/* TODO: something */
			throw new SPOException(e);
		} catch (UnsupportedEncodingException e)
		{
			/* TODO: something */
			throw new SPOException(e);
		} catch (IOException e)
		{
			/* TODO: something */
			throw new SPOException(e);
		}

		return resultSet;
	}

	public static int doCount(URL lurlFedoraAPIA, String lstrSubject,
			String lstrPredicate, String lstrObject) throws SPOException
	{
		String queryBase = "type=triples&lang=spo&format=N-Triples&stream=on&query=";

		lstrObject = checkObject(lstrObject);

		String queryString = (lstrSubject == null ? "*" : lstrSubject) + " "
				+ (lstrPredicate == null ? "*" : lstrPredicate) + " "
				+ (lstrObject == null ? "*" : lstrObject);

		URL searchURL;
		BufferedReader content;
		int lintCount = 0;
		try
		{
			searchURL = new URL(lurlFedoraAPIA.getProtocol(), lurlFedoraAPIA
					.getHost(), lurlFedoraAPIA.getPort(), Constants.RESOURCE_INDEX_URL
					+ queryBase + URLEncoder.encode(queryString, "US-ASCII"));

			content = new BufferedReader(
					new InputStreamReader(searchURL.openStream()));

			for (String line = content.readLine(); line != null; line = content
					.readLine())
			{
				lintCount++;
				// resultSet.add(readNTriple(line));
			}

			content.close();

		} catch (MalformedURLException e)
		{
			/* TODO: something */
			throw new SPOException(e);
		} catch (UnsupportedEncodingException e)
		{
			/* TODO: something */
			throw new SPOException(e);
		} catch (IOException e)
		{
			/* TODO: something */
			throw new SPOException(e);
		}

		return lintCount;
	}

	public static Set<Triple> doAnd(URL lurlFedoraAPIA, Set<Triple> lsetResult,
			String lstrPredicate, String lstrObject)
			throws SPOException
	{
		String queryBase = "type=triples&lang=spo&format=N-Triples&stream=on&query=";

		lstrObject = checkObject(lstrObject);

		HashSet<Triple> resultSet = new HashSet<Triple>();

		URL searchURL;
		BufferedReader content;

		Iterator<Triple> liter = lsetResult.iterator();

		//  For each result item, do the search
		while (liter.hasNext())
		{
			
			String lstrSubject = "<" + liter.next().subject + ">";

			String queryString = lstrSubject + " "
					+ (lstrPredicate == null ? "*" : lstrPredicate) + " "
					+ (lstrObject == null ? "*" : lstrObject);

			try
			{
				searchURL = new URL(lurlFedoraAPIA.getProtocol(), lurlFedoraAPIA
						.getHost(), lurlFedoraAPIA.getPort(), Constants.RESOURCE_INDEX_URL
						+ queryBase + URLEncoder.encode(queryString, "US-ASCII"));

				content = new BufferedReader(new InputStreamReader(searchURL
						.openStream()));

				for (String line = content.readLine(); line != null; line = content
						.readLine())
				{
					Triple lobjTrip = readNTriple(line);
					if (lobjTrip.isSubjectContainedIn(lsetResult))
					{
						resultSet.add(lobjTrip);
					}
				}

				content.close();

			} catch (MalformedURLException e)
			{
				/* TODO: something */
				throw new SPOException(e);
			} catch (UnsupportedEncodingException e)
			{
				/* TODO: something */
				throw new SPOException(e);
			} catch (IOException e)
			{
				/* TODO: something */
				throw new SPOException(e);
			}
		}

		return resultSet;

	}

	/**
	 * Turns a spo query returned line into a Triple
	 * 
	 * @param nTripleLine
	 * @return
	 */
	private static Triple readNTriple(String nTripleLine)
	{

		// Split length tells if object value is PID or string value:
		// length=2 -> string value
		// length=3 -> pid value
		if (nTripleLine.length() < 2)
		{
			throw new RuntimeException();
		}

		// Strip the trailing space-period that fedora seems to want to add
		if (nTripleLine.endsWith(" ."))
		{
			nTripleLine = nTripleLine.substring(0, (nTripleLine.length() - 2));
		}

		String[] lineParts = nTripleLine.split("> <");
		String subject = null;
		String pred = null;
		String object = null;

		if (lineParts.length == 2)
		{
			subject = lineParts[0].replaceAll("<", "").replaceAll(">", "");
			String[] tailParts = lineParts[1].split("> \"");
			if (tailParts.length == 2)
			{
				pred = tailParts[0].replaceAll("<", "").replaceAll(">", "");
				object = tailParts[1].replaceAll("\"", "");
			} else
			{
				/* TODO: throw something */
				throw new RuntimeException();
			}
		} else if (lineParts.length == 3)
		{
			subject = lineParts[0].replaceAll("<", "").replaceAll(">", "");
			pred = lineParts[1].replaceAll("<", "").replaceAll(">", "");
			object = lineParts[2].replaceAll("<", "").replaceAll(">", "");
		} else
		{
			/* TODO: throw something */
			throw new RuntimeException();
		}

		if (subject == null || pred == null || object == null)
		{
			throw new RuntimeException();
		}

		return new Triple(subject, pred, object);
	}

	// }

	/**
	 * Describes a Triple
	 */
	public static class Triple
	{
		public final String subject;

		public final String predicate;

		public final String object;

		public boolean isSubjectContainedIn(Set<Triple> inSet)
		{
			Iterator<Triple> literSet = inSet.iterator();
			while (literSet.hasNext())
			{
				if (literSet.next().subject.equals(this.subject))
				// && lobjTrip.predicate == this.predicate
				// && lobjTrip.object == this.object)
				{
					return true;
				}
			}

			return false;
		}

		public Triple(String subject, String pred, String object)
		{
			if (subject == null || pred == null || object == null)
			{
				throw new NullPointerException("Cannot have null element "
						+ "in triple '" + subject + ", " + pred + ", " + object + "'");
			}
			this.subject = subject;
			this.predicate = pred;
			this.object = object;
		}
	}

	private static String checkObject(String lstrObject)
	{
		// Don't do nulls.
		if (lstrObject == null)
			return lstrObject;

		// Ok, here's why it's called convenience...
		if (!(lstrObject.startsWith("<")) && !(lstrObject.startsWith("\"")))
		{
			// forgot to wrap the object, default to quoted string.
			lstrObject = '"' + lstrObject + '"';
		}

		return lstrObject;

	}

}
