/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model;

/**
 * Represents a the basic elements that compose an NDR object.
 * 
 * @author birkland
 * @since 1.0
 */
public interface Component {

    /** The type of component. */
    public Type getType();

    /** The object that contains the component. */
    public NDRObjectInfo subject();

    /** Specifies the class of component in an NDR object. */
    public static enum Type {
        /** Property component, See {@link Property} */
        Property,

        /** Datastream component, See {@link Datastream} */
        Datastream,

        /** Relationship component, See {@link Relationship} */
        Relationship,
    }
    
    /**
     * Pattern that can match specific component instances.
     * 
     * @author birkland
     * @since 1.0
     */
    public interface Pattern {

        /** Get the type of component that can possibly match this pattern. */
        public Component.Type getMatchingType();

        /** Determine if a particular component instance matches this pattern. */
        public boolean matches(Component c);
    }
    
    public interface Name<T> {}

}
