/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.content;

/**
 * Marker for classes that contain definitions of object components.
 * <p>
 * Classes are marked as Definitions if they contain any number of public final
 * {@link org.nsdl.repository.model.Property.Name} or
 * {@link org.nsdl.repository.model.Relationship.Name} constants that are meant
 * to comprise a definition of a particular class, model, or set of
 * characteristics of an NDR object. Any class that is marked as Definitions may
 * have its declared component names accessible as a {@link java.util.Set} by
 * using reflection. For example:
 * </p>
 * <p>
 * <pre>
 * public interface MyModel extends Definitions {
 *      public static final Property.Name STATUS = 
 *          new Property.Name(myNamespace, "status", prefix);
 *      
 *      public static final Property.Name ID =
 *          new Property.Name(myNamespace, "id", prefix);
 *          
 *      .
 *      .
 *      .
 * }
 * </pre>
 * </p>
 * <p>
 * This class then may be used as follows, for example:
 * </p>
 * <p>
 * <pre>
 *      Set&lt;Property.Name&gt; myModelProperties = 
 *          ContentUtil.getPropertyNames(MyModel.class, Scope.All);
 *      .
 *      .
 * </pre>
 * </p>
 * 
 * @see ModelUtil#getPropertyNames(Class, Scope)
 * @see ModelUtil#getRelationshipNames(Class, Scope)
 * @author birkland
 * @since 1.1
 */
public interface Definitions {
}
