/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.impl;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URL;

import org.nsdl.repository.access.AccessException;
import org.nsdl.repository.model.Component;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.util.StreamUtil;

/**
 * Simple datastream implementation.
 * <p>
 * Effectively, this is just a bean. You get out only what you put in.
 * <p>
 * 
 * @author birkland
 * 
 */
public class BasicDatastream implements Datastream {

    private byte[] m_bytes;
    private final Name m_name;
    private String m_mime;
    private long m_size = 0;
    private URL m_url;
    private String m_fileName;

    public BasicDatastream(Component.Name<Datastream> name) {
        if (!(name instanceof Name)) {
            throw new IllegalArgumentException("Expected name to be "
                    + Name.class.getName() + ", but instead got a "
                    + name.getClass().getName());
        }

        m_name = (Name) name;
    }

    public InputStream getContent() throws AccessException {
        if (m_bytes != null) {
            return new ByteArrayInputStream(m_bytes);
        } else
            return null;
    }

    public InputStream getContent(String version) throws AccessException {
        return getContent();
    }

    public String getMimeType() {
        return m_mime;
    }

    public Name getName() {
        return m_name;
    }

    public void setSize(long size) {
        m_size = size;
    }

    public long getSize() {
        return m_size;
    }

    public URL getURL() {
        return m_url;
    }

    public void setContent(InputStream content) {
        m_bytes = StreamUtil.streamToByteArray(content);
    }

    public void setMimeType(String mime) {
        m_mime = mime;

    }

    public void setURL(URL redirect) {
        m_url = redirect;
    }

    public boolean isCanonical() {
        if (m_bytes != null) {
            return true;
        } else {
            return false;
        }
    }

    public String getLabel() {
        return m_fileName;
    }

    public void setLabel(String name) {
        m_fileName = name;
    }
}
