/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.impl;

/**
 * Allows for actions on an object's components (adds, deletes, etc) to be
 * monitored.
 * 
 * @author birkland
 * 
 * @param <T>
 *            Type of components of which events will be listened for
 */
public interface MutationListener<T> {

    /**
     * Notify of an add
     * 
     * @param addedElement
     *            the added element
     */
    public void notifyAdd(T addedElement);

    /**
     * Notify of a removal
     * 
     * @param removedElement
     *            the removed element
     */
    public void notifyRemove(T removedElement);

    /**
     * Notify of an element that has been replaced
     * 
     * @param updated element that has been updated;
     */
    public void notifyUpdate(T updated);
}
