/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.types;

import org.nsdl.repository.access.Results;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.content.AggregatorModel;

/**
 * Branded grouping of NCore objects.
 * 
 * @author birkland
 * @since 1.2
 */
public interface Aggregator extends Grouping, AggregatorModel.Components {

    /**
     * {@linkplain org.nsdl.repository.model.content.AggregatorModel.Components#ASSOCIATED_WITH Associate}
     * a resource with the given Aggregator.
     * 
     * If there is a Resource already associated with this Aggregator, it will
     * be unassociated and replaced with the given Resource.
     * 
     * @param resource
     *            Resource to associate.
     */
    public void setAssociatedResource(Resource resource);

    /**
     * Get the resource associated with this Aggregator, if present.
     * 
     * @return Associated resource, null if there are none.
     */
    public Resource getAssociatedResource();

    /**
     * Add an object as a member of this Aggregator.
     * 
     * Adding a member through this method will not be reflected in the
     * repository until {@link #commit()} is called.
     * 
     * @param toAdd
     *            Identity of the object to add.
     */
    public void addMember(NDRObjectInfo toAdd);

    /**
     * Get all members that are of the given type.
     * 
     * This will return only those members that are of the supplied type. If the
     * given type is null, then all members will be returned, which is the same
     * effect as calling {@link #getMembers()}
     * 
     * @param <T>
     *            Concrete member type.
     * @param type
     *            Type of member to be retrieved.
     */
    public <T extends NDRObject> Results<T> getMembers(final Type<T> type);

    /** {@inheritDoc} */
    public <T extends NDRObject> long countMembers(final Type<T> type);
}
