/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.types;
import org.nsdl.repository.model.NDRObject;

interface ConcreteObject extends  NDRObject {

    /**
     * Determine if the object is directly or indirectly a member of the given
     * <code>Aggregator</code>.
     * 
     * Indirect membership, as used here, means that an object is member of an
     * Aggregator, or a member of one of its members, or a member of a member of
     * one of its members, and so on. For more information, please refer to <a
     * href="http://wiki.nsdl.org/index.php/Community:NCore/Membership_and_Projection">
     * the online documentation</a>.
     * 
     * 
     * @param aggregator
     *            Aggregator that defines a region in the repository that may
     *            contain this object.
     * @return true if the object is underneath the supplied Aggregator.
     */
    public boolean isUnder(Aggregator aggregator);
}
