/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */

package org.nsdl.repository.model.types;

import java.io.InputStream;
import java.util.HashSet;
import java.util.Set;

import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.model.ExistenceException;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.types.Type;

/**
 * Represents a set of assertions/statements about NCore objects.
 * 
 * @author birkland
 * @since 1.2
 */
class GenericMetadataImpl extends ConcreteObjectBase implements Metadata {

    private final NDRAccess m_ndr;

    public GenericMetadataImpl(NDRObject identity, NDRAccess repository)
            throws ExistenceException {
        super(identity, repository);
        if (!Type.Metadata.equals(this.type())) {
            throw new ExistenceException(identity.handle()
                    + " is not a Metadata object");
        }
        m_ndr = repository;
    }

    /** {@inheritDoc} */
    public InputStream getMetadata(String format) {
        Datastream mdStream = getFormatDS(format);

        if (mdStream == null)
            return null;

        return mdStream.getContent();
    }

    /** {@inheritDoc} */
    public Set<String> getFormats() {
        HashSet<String> ids = new HashSet<String>();
        for (Datastream d : getDatastreams()) {
            if (d.getName().getID().startsWith("format_")) {
                ids.add(d.getName().getID().replaceFirst("format_", ""));
            }
        }

        return ids;
    }

    /** {@inheritDoc} */
    public void setMetadata(String format, InputStream content) {

        Datastream metadata = getFormatDS(format);

        if (metadata == null) {
            Datastream.Name name = new Datastream.Name("format_" + format);
            metadata = addDatastream(name);
            metadata.setMimeType("application/xml");
        }
        metadata.setContent(content);
    }

    /** {@inheritDoc} */
    public String getMimeType(String format) {
        Datastream ds = getFormatDS(format);
        if (format == null) {
            return null;
        } else {
            return ds.getMimeType();
        }
    }

    /** {@jnheritDoc} */
    public void setMimeType(String format, String mimetype) {
        Datastream ds = getFormatDS(format);
        if (format != null) {
            ds.setMimeType(mimetype);
        }
    }

    public MetadataProvider getProvider() {
        return m_ndr.getObject(getRelationship(Metadata.METADATA_PROVIDED_BY)
                .related(), Type.MetadataProvider);
    }

    private Datastream getFormatDS(String format) {
        Datastream.Name dsName = new Datastream.Name("format_" + format);
        return getDatastream(dsName);
    }
}
