/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.types;

import java.io.InputStream;
import java.util.Set;

import org.nsdl.repository.model.content.MetadataModel;

/**
 * Represents a set of assertions/statements about NCore objects.
 * 
 * @author birkland
 * @since 1.2
 */
public interface Metadata extends MetadataModel.Components, ConcreteObject {

    /** <code>nsdl_dc</code> Qualified Dublin Core metadata format name. */
    public static final String FORMAT_NSDL_DC = "nsdl_dc";
    public static final String FORMAT_OAI_DC = "oai_dc";

    /**
     * Get a Metadata stream from the object in a particular format, if defined.
     * 
     * @param format
     *            Metadata format name (e.g. <code>nsdl_dc</code>)
     * @return Stream of metadata, if defined. <code>null</code> otherwise.
     */
    public InputStream getMetadata(String format);

    /**
     * Get the names of all metadata formats available in this object.
     * 
     * @return Set of all available format names.
     */
    public Set<String> getFormats();

    /**
     * Set the metadata payload of the object in a given format.
     * 
     * @param format
     *            Format of metadata (e.g. <code>nsdl_dc</code>)
     * @param content
     *            Stream of data comprising the Metadata record.
     */
    public void setMetadata(String format, InputStream content);

    /**
     * Set the MIME type for a particular metadata format.
     * 
     * @param format
     *            Format of metadata of which to set the MIME type.
     * @param mimetype
     *            MIME type.
     */
    public void setMimeType(String format, String mimetype);

    /**
     * Get the MIME type for a particular metadata format.
     * 
     * @param format
     *            Format of metadata of which to get the MIME type.
     * @return MIME type as a string.
     */
    public String getMimeType(String format);
    
    
    /** MetadataProvider that provided this Metadata */
    public MetadataProvider getProvider();

}
