/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.types;

/*
 * -----------------------------------------------------------------------------
 * 
 * <p><b>License and Copyright: </b>The contents of this file are subject to
 * the Educational Community License (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of the License
 * at <a href="http://www.opensource.org/licenses/ecl1.txt">
 * http://www.opensource.org/licenses/ecl1.txt.</a></p>
 * 
 * <p>Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License.</p>
 * 
 * <p>The entire file consists of original code. Copyright &copy; 2004-2007
 * Cornell University. All rights reserved.</p>
 * 
 * -----------------------------------------------------------------------------
 */

import org.nsdl.repository.access.Results;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.content.MetadataProviderModel;

/**
 * Branded source of {@linkplain Metadata} in the NCore model.
 * 
 * @author birkland
 * @since 1.2
 */
public interface MetadataProvider extends Grouping, MetadataProviderModel.Components {

    /**
     * Create new Metadata that describes a given object.
     * 
     * Will pre-populate the Metadata object with the correct relationships to
     * this provider and the described object (Resource or Aggregator).
     * 
     * The new Metadata object have an empty payload (i.e. it will have no
     * default DublinCore content) .
     * 
     * @param describedObject
     *            Identity of the object that will be described by the given
     *            Metadata object.
     * 
     * @return Minimally populated Metadata object.
     */
    public Metadata newMetadata(NDRObjectInfo describedObject);

    public Results<Metadata> getMembers();

}
