/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.oai;

import java.io.InputStream;

import org.apache.log4j.Logger;
import org.nsdl.util.xml.NDRXMLutil;
import org.w3c.dom.Document;

public class InfoStream extends XMLContainer {

    private static final Logger logger = Logger.getLogger(InfoStream.class);

    private static final String CATEGORY_ELEMENT = "nsdlAboutCategory";
    private static final String LINK_ELEMENT = "link";
    private static final String DEFAULT_CONTENTS =
            "<info>\n"
                    + " <nsdlAboutCategory></nsdlAboutCategory>\n"
                    + " <link></link>"
                    + " <harvestDate>1970-01-01T00:00:00Z</harvestDate>\n"
                    + " <harvestHarvestType>OAI2.0</harvestHarvestType>\n"
                    + " <harvestDatasource>http://ndr.nsdl.org</harvestDatasource>\n"
                    + " <metadataNamespace>http://www.openarchives.org/OAI/2.0/oai_dc/</metadataNamespace>\n"
                    + " <harvestIdentifier>oai:example.org:null</harvestIdentifier>\n"
                    + " <harvestRecordDatestamp>1970-01-01T00:00:00Z</harvestRecordDatestamp>\n"
                    + " <repositoryPrimaryIdentifier>http://example.org</repositoryPrimaryIdentifier>\n"
                    + " <harvestDatasourcePublic>true</harvestDatasourcePublic>\n"
                    + "</info>";

    public InfoStream() {
        super();
    }

    public InfoStream(InputStream initial) {
        super(initial);
    }

    public void setCategory(String categoryValue) {
        setValue(CATEGORY_ELEMENT, categoryValue);
    }

    public void setLink(String linkValue) {
        setValue(LINK_ELEMENT, linkValue);
    }

    public String getLink() {
        try {
            return getValue(LINK_ELEMENT);
        } catch (Exception e) {
            logger.info("Could not get link content");
            return "";
        }
    }

    Document getDefault() {
        try {
            return NDRXMLutil.getDocument(DEFAULT_CONTENTS, false, false);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    Logger getLogger() {
        return logger;
    }
}
