package org.nsdl.repository.oai;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import org.apache.log4j.Logger;
import org.nsdl.util.xml.NDRXMLutil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public abstract class XMLContainer {
    private final Document m_xmlDom;

    public XMLContainer() {
        m_xmlDom = getDefault();
    }

    public XMLContainer(InputStream initial) {
        m_xmlDom = getDefault();
        try {
            init(NDRXMLutil.getDocument(initial, false, false)
                    .getDocumentElement());
        } catch (Exception e) {
            Logger log = getLogger();
            if (log.isDebugEnabled()) {
                log.debug("Could not read XML contents", e);
            } else {
                log.warn("Could not read XML content - using default value");
            }
        }
    }

    public InputStream getContents() {
        try {
            getLogger().trace(
                    "Returning info contents as"
                            + NDRXMLutil.serializeDOMdoc(m_xmlDom).replaceAll(
                                    "\\s*<\\?xml.*?\\?>\\s*", ""));
            return new ByteArrayInputStream(NDRXMLutil
                    .serializeDOMdoc(m_xmlDom).replaceAll(
                            "\\s*<\\?xml.*?\\?>\\s*", "").getBytes("UTF-8"));

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public void setValue(String element, String value) {
        Element el = getElement(element);
        if (el != null) {
            getLogger().trace(
                    "Setting info " + element + " (formerly "
                            + NDRXMLutil.getElementValue(el) + ") to " + value);
            NDRXMLutil.setElementValue(el, value);
        }
    }

    public String getValue(String name) {
        if (getElement(name) != null) {
            return getElement(name).getTextContent();
        } else {
            return null;
        }
    }

    abstract Document getDefault();

    abstract Logger getLogger();

    private void init(Element content) {
        NodeList parts = content.getChildNodes();
        for (int i = 0; i < parts.getLength(); i++) {
            Node part = parts.item(i);
            if (part.getNodeType() == Node.ELEMENT_NODE) {

                if (!hasChildElements((Element) part)) {
                    String tag = ((Element) part).getTagName();

                    Element existing = NDRXMLutil.getChildElement(content,
                            null, tag);

                    if (existing != null) {
                        setValue(tag, existing.getTextContent());
                    }
                } else {
                    init((Element) part);
                }
            }
        }
    }

    private boolean hasChildElements(Element initial) {
        NodeList parts = initial.getChildNodes();
        for (int i = 0; i < parts.getLength(); i++) {
            Node part = parts.item(i);
            if (part.getNodeType() == Node.ELEMENT_NODE) {
                return true;
            }
        }

        return false;
    }

    private Element getElement(String tagName) {
        NodeList elements = m_xmlDom.getElementsByTagName(tagName);
        if (elements.getLength() == 1) {
            return (Element) elements.item(0);
        } else {
            return null;
        }
    }
}
