/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.util;

import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.httpclient.methods.multipart.PartSource;
import org.nsdl.repository.access.AccessException;
import org.nsdl.repository.error.RepositoryException;
import org.nsdl.repository.model.Datastream;

/** Represents a datastream payload in a multi-part form
 * <p>
 * Datastreams may be represented as 'files' in a multipart form
 * both in the NDR api and in the Fedora API-M.  This 
 * convenieltly wraps a datastream for inclusion in a 
 * multipart form.
 * </p>
 * @author birkland
 *
 */
public class DatastreamPartSource implements PartSource {

    private final Datastream datastream;
    private final Datastream.Name name;
    
    public DatastreamPartSource(Datastream ds) {
        this.datastream = ds;
        this.name = ds.getName();
    }
    
    /** {@inheritDoc} */
    public InputStream createInputStream() throws IOException {
        try {
            return datastream.getContent();
        } catch (AccessException e) {
            throw new RepositoryException(
                    "Could not get datastream content", e);
        }
    }

    /** {@inheritDoc} */
    public String getFileName() {
        return name.getID();
    }

    /** {@inheritDoc} */
    public long getLength() {
        return datastream.getSize();
    }

}
