/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.util;

/** Utility class for encoding/decoding special characters. */
public class Encoder {

    /**
     * Encode text into an xml-friendly form.
     * <p>
     * Replaces any 'special' characters that must be encoded as entity
     * references when represented in xml; In particular, it substitutes entity
     * references for the following characters: <code>&quot;</code>,
     * <code>&amp;</code>, <code>&apos;</code>, <code>&lt;</code>,
     * <code>&gt;</code>.
     * </p>
     * 
     * @param value
     *            Characters to be encoded.
     * @return Encoded character sequence.
     */
    public static CharSequence xmlEncode(CharSequence value) {
        StringBuilder encoded = new StringBuilder(value.length() + 20);
        for (int i = 0; i < value.length(); i++) {
            char c = value.charAt(i);
            switch (c) {
                case '"':
                    encoded.append("&quot;");
                    break;
                case '&':
                    encoded.append("&amp;");
                    break;
                case '\'':
                    encoded.append("&apos;");
                    break;
                case '<':
                    encoded.append("&lt;");
                    break;
                case '>':
                    encoded.append("&gt;");
                    break;
                default:
                    encoded.append(c);
            }
        }
        return encoded;
    }
}
