/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.util;

import java.util.Arrays;
import java.util.Date;

import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.Relationship;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.model.impl.MutableObjectBase;

/**
 * Blank object template
 * <p>
 * Useful for describing object contents or templating. It's just a blank, empty
 * object that can have arbitrary components added or removed, which can be
 * useful for various purposes, such as representing an object structure or
 * template
 * </p>
 * 
 * @author birkland
 * 
 */
public class NDRObjectTemplate extends MutableObjectBase {

    /**
     * Create a new, empty template
     * 
     * @param type
     *            The type of object that this template represents
     */
    public NDRObjectTemplate(Type<? extends NDRObject> type) {
        initNew(type, null);
    }


    public NDRObjectTemplate(Type<? extends NDRObject> type, String handle) {
        initNew(type, handle);
    }

    public Date getLastModifiedDate() {
        return null;
    }

    public Date getCreatedDate() {
        return null;
    }

    public void commit() {
        throw new UnsupportedOperationException(
                "Template objects' can't commit!");
    }

    public void addExternalRelationship(NDRObjectInfo subject,
            Relationship.Name predicate) {
        addRelationships(Arrays.asList(new Relationship(subject, predicate,
                this)));
    }

}
