/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.util;

import java.util.Collection;

import org.nsdl.repository.access.AccessException;
import org.nsdl.repository.model.Datastream;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.model.Relationship;
import org.nsdl.repository.model.Datastream.Name;

/**
 * Base class for objects that are intended to be purely read-only.
 * <p>
 * Any CUD methods will return an UnsupportedOperationException.
 * </p>
 * 
 * @author birkland
 * 
 */
public abstract class ReadOnlyObject implements NDRObject {

    public Datastream addDatastream(Name arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void addDatastreams(Collection<Datastream> arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void addProperties(Collection<Property> arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void addProperty(org.nsdl.repository.model.Property.Name arg0,
            String arg1) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void addRelationship(
            org.nsdl.repository.model.Relationship.Name arg0, NDRObjectInfo arg1) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void addRelationships(Collection<Relationship> arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void commit() throws AccessException {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void removeDatastream(Name arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void removeDatastreams(Collection<Datastream> arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void removeProperties(Collection<Property> arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void removeProperty(org.nsdl.repository.model.Property.Name arg0,
            String arg1) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void removeRelationship(
            org.nsdl.repository.model.Relationship.Name arg0, NDRObjectInfo arg1) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void removeRelationships(Collection<Relationship> arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void setHandle(String arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }

    public void setState(State arg0) {
        throw new UnsupportedOperationException("Object is read-only");
    }
}
