/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.util;

import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.types.Type;


/** Static representation of an NDR object
 * 
 * This class is mainly intended to represent NDR object information 
 * statically, i.e. used as identifying placeholders.  Consequently, 
 * information in a StaticObjectInfo is not tied to the repository, 
 * which may be desirable
 *  
 * 
 * @author birkland
 *
 */
public class StaticObjectInfo implements NDRObjectInfo {

	private final Type<? extends NDRObject> type;
	private final String handle;
	
    public StaticObjectInfo(String handle) {
        this.handle = handle;
        this.type = null;
    }
    
    public StaticObjectInfo(Type<? extends NDRObject> type) {
        this.type = type;
        this.handle = null;
    }
    
	public StaticObjectInfo(String handle, Type<? extends NDRObject> type) {
		this.type = type;
		this.handle = handle;
	}
	
    /** {@inheritDoc} */
	public String handle() { 
		return this.handle;
	}
	
    /** {@inheritDoc} */
	public Type<? extends NDRObject> type() {
		return this.type;
	}
}
