/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Servlet for returning raw text/binary content to the client.
 * 
 * <p>
 * May be defined as a 'forward' in struts. To use, two tasks must be done in
 * advance:
 * <ul>
 * <li>Create an InputStream containing the content to serve, and store it as an
 * attribute in the HttpServletRequest named 'contentStream</li>
 * <li>Define the mime-type in a HttpServletRequest named 'contentType'</li>
 * </ul>
 * 
 * The contents of the ByteArrayOutputStream will then be sent to the clent
 */
public class BinaryStreamlet extends HttpServlet {
    private static final long serialVersionUID = 1;
    private static final int BUF_SIZE = 4096;

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        response.reset();
        /* Get the content type - default to text/plain if undefined */
        String contentType = (String) request.getAttribute("contentType");
        if (contentType == null || "".equals(contentType)) {
            contentType = "application/octet-stream";
        }

        response.setContentType(contentType);
        String label = (String) request.getAttribute("label");
        if (label != null && !"".equals(label)) {
            response.setHeader("Content-Disposition", "inline; filename=\""
                    + label + "\"");
        }

        OutputStream out = response.getOutputStream();
        InputStream contentStream = (InputStream) request
                .getAttribute("contentStream");

        byte[] buf = new byte[BUF_SIZE];

        try {
            for (int n; (n = contentStream.read(buf)) != -1;) {
                out.write(buf, 0, n);
            }
        } catch (Throwable e) {
            /* Probably interrupted socket from client terminaiton. Ignore */
        } finally {
            try {
                contentStream.close();
            } catch (Exception e) {
                /* Nothing */
            }

            try {
                out.close();
            } catch (Exception e) {
                /* Nothing */
            }
        }
    }
}
