/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.util.security;

/** Convenience class for formatting base64-encoded public and private key data
 * for inclusion in various ASCII formats.  
 * 
 * @author birkland
 *
 */
class KeyStringFormatter {
	private static int DEFAULT_LINE_LENGTH = 48;
	
	/** Formats a single line string into multi-line chunk 
	 * 
	 * Prepends &lt;tabStop&gt; number of tabs before each line,
	 * and limits line length to a KeyStringFormatter.DEFAULT_LINE_LENGTH
	 * before moving to a new line
	 * 
	 * @param inputString String to be formatted
	 * @param tabStop Number of tabs to place before tach line
	 * @return formatted string
	 */
	public static String format(String inputString, int tabStop) {
		return format(inputString, tabStop, DEFAULT_LINE_LENGTH);
		
	}
	
	/** Formats a single line string into multi-line chunk
	 * 
	 * Prepends &lt;tabStop&gt; number of tabs before each line,
	 * and limits line length to a &lt;lineLength&gt;
	 * before moving to a new line
	 * 
	 * @param inputString String to be formatted
	 * @param tabStop Number of tabs to place before each line
	 * @param lineLength Maximum line length
	 * @return formatted string
	 */
	public static String format(String inputString, int tabStop, int lineLength) {
		StringBuilder output = new StringBuilder();
		StringBuilder tabs = new StringBuilder();
		for (int t = 0; t < tabStop; t++) {tabs.append("\t");}
		
			for (int i = 0; i < inputString.length(); i += lineLength) {
			int start = i;
			int end = start + lineLength;
			
			if (end >= inputString.length()) {
				end = inputString.length();
			}
			
			output.append(tabs + inputString.substring(start, end) + "\n");
		}
			
			return output.toString();
	}
}
