/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.util.security;

import java.security.KeyException;
import java.security.PrivateKey;
import java.security.PublicKey;

/** Interface for serializing/deserializing keys in particular formats
 * 
 * @author birkland
 *
 */
public interface KeyTranslator {
	
	/** Encodes (serializes) a public key
	 * 
	 * @param key THe Public Key
	 * @return String of key in encoded form
	 * @throws KeyException
	 */
	public String encodePublicKey(PublicKey key) 
			throws KeyException;
	
	/** Encodes (serializes) a private key
	 * 
	 * @param key The private Key
	 * @return String of key in encoded form
	 * @throws KeyException
	 */
	public String encodePrivateKey(PrivateKey key) 
			throws KeyException;
	
	/** Encodes (serializes) an encrypted private key
	 * 
	 * @param key The private Key
	 * @return String of key in encoded form
	 * @throws KeyException
	 */
	public String encodePrivateKey(PrivateKey key, 
			String passphrase) throws KeyException;
	
	/** Decodes (reads in) a PublicKey.
	 * 
	 * @param source Encoded representation of the key
	 * @return Public key
	 * @throws KeyException
	 */
	public PublicKey decodePublicKey(String source) 
			throws KeyException;
	
	/** Decodes (reads in) a PrivateKey
	 * 
	 * @param source Encoded representation of the key
	 * @return PrivateKey
	 * @throws KeyException
	 */
	public PrivateKey decodePrivateKey(String source) 
			throws KeyException;
	
	/** Decodes (reads in) an encrypted PrivateKey
	 * 
	 * @param source Encoded representation of the key
	 * @param passphrase Decryption pasphrase for the encrypted key
	 * @return PrivateKey
	 * @throws KeyException
	 */
	public PrivateKey decodePrivateKey( 
			String source, String passphrase) throws KeyException;
}
