/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.access;

import org.junit.BeforeClass;
import org.junit.Test;
import org.nsdl.repository.access.filters.FilterContext;
import org.nsdl.repository.access.filters.FilteredNDRAccess;
import org.nsdl.repository.access.filters.NDRAccessFilter;
import org.nsdl.repository.impl.fedora.MockFedoraNDRAccess;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.types.Type;

import static junit.framework.Assert.assertTrue;

public class FilteredNDRAccessTest {

    private static NDRAccess ndr;

    @BeforeClass
    public static void setup() {
        ndr = MockFedoraNDRAccess.getInstance();
    }

    /** Assures that the filters run at all */
    @Test
    public void filterTriggerTest() throws Exception {
        FilteredNDRAccess filterNDR = new FilteredNDRAccess(ndr);
        FilterLogger logger = new FilterLogger();
        filterNDR.addFilter(logger);

        NDRObject newObject = filterNDR.newObject(Type.Resource);
        newObject.commit();
        assertTrue("Filter was never called! ", logger.triggered);
    }

    private class FilterLogger implements NDRAccessFilter {
        private boolean triggered = false;

        public void doFilter(NDRObject object, FilterContext context) {
            triggered = true;
        }

        public void init(NDRAccess ndr) {
        }

    }
}
