/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.access.filters;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.impl.fedora.MockFedoraNDRAccess;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.types.Type;

import static org.nsdl.repository.model.content.OAIModel.Item.Components.ITEM_ID;

public class NSDLDefaultItemIdFilterTest {
    private static NDRAccess ndr;
    private static NSDLDefaultItemIdFilter filter;

    @BeforeClass
    public static void setUp() {
        filter = new NSDLDefaultItemIdFilter();
        ndr = MockFedoraNDRAccess.getInstance();
    }

    @Test
    public void starAssignemtTest() {
        NDRObject o = ndr.newObject(Type.Metadata);
        o.addProperty(ITEM_ID, "*");

        MockFilterContext context = new MockFilterContext(
                FilterState.BEFORE_COMMIT);

        filter.doFilter(o, context);

        Assert.assertFalse("Filter did not replace itemId marker", o
                .getProperty(ITEM_ID).value().equals("*"));
    }

    @Test
    public void emptyAssignemtTest() {
        NDRObject o = ndr.newObject(Type.Metadata);
        o.addProperty(ITEM_ID, "");

        MockFilterContext context = new MockFilterContext(
                FilterState.BEFORE_COMMIT);

        filter.doFilter(o, context);

        Assert.assertFalse("Filter did not replace itemId marker", o
                .getProperty(ITEM_ID).value().equals(""));
    }

    @Test
    public void dontMessWithExistingItemIDTest() {
        final String EXISTING_ID = "oai:test";
        NDRObject o = ndr.newObject(Type.Metadata);
        o.addProperty(ITEM_ID, EXISTING_ID);

        MockFilterContext context = new MockFilterContext(
                FilterState.BEFORE_COMMIT);

        filter.doFilter(o, context);

        Assert.assertTrue("Filter should not have changed the itemId", o
                .getProperty(ITEM_ID).value().equals(EXISTING_ID));
    }

    @Test
    public void dontDieIfNoItemidTest() {
        NDRObject o = ndr.newObject(Type.Metadata);

        MockFilterContext context = new MockFilterContext(
                FilterState.BEFORE_COMMIT);

        filter.doFilter(o, context);

        /* We should get here without an exception being thrown */

        Assert.assertNull("Should not have an ItemID property!", o
                .getProperty(ITEM_ID));
    }
}
