/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.action;


/**
 * provide valid action paths as a typesafe enum pattern 
 *   per _Effective Java_ by Joshua Bloch, Item 21, pg 104
 *
 * These values should EXACTLY match those indicated as path attributes on 
 * action elements in action mappings in the struts-config.xml file.
 * 
 * @author Naomi Dushay
 */
public class ActionPathType
{
  private final String path;
  
  /**
   * private constructor ensures that objects of this
   * class cannot be created or extended, so
   * only the public static final fields are available.
   */
  private ActionPathType(String type)
  {
    this.path = type;
  }

  public String toString()
  {
    return path;
  }
  
  // these values should EXACTLY match those indicated as path attributes on 
  // action elements in action mappings in the struts-config.xml file.

  public static final ActionPathType ADD_AGENT = new ActionPathType("/addAgent");
  public static final ActionPathType ADD_AGGREGATOR = new ActionPathType("/addAggregator");
  public static final ActionPathType ADD_METADATA = new ActionPathType("/addMetadata");
  public static final ActionPathType ADD_METADATA_PROVIDER = new ActionPathType("/addMetadataProvider");
  public static final ActionPathType ADD_RESOURCE = new ActionPathType("/addResource");

  public static final ActionPathType COUNT_MEMBERS = new ActionPathType("/countMembers");
  
  public static final ActionPathType DESCRIBE = new ActionPathType("/describe");

  public static final ActionPathType FIND = new ActionPathType("/find");
  public static final ActionPathType FIND_AGENT = new ActionPathType("/findAgent");
  public static final ActionPathType FIND_METADATA = new ActionPathType("/findMetadata");
  public static final ActionPathType FIND_RESOURCE = new ActionPathType("/findResource");

  public static final ActionPathType GET = new ActionPathType("/get");
  public static final ActionPathType GET_CONTENT = new ActionPathType("/getContent");
  public static final ActionPathType GET_METADATA = new ActionPathType("/getMetadata");
  public static final ActionPathType GET_METADATA_ABOUT = new ActionPathType("/getMetadataAbout");

  public static final ActionPathType LIST_MEMBERS = new ActionPathType("/listMembers");
  public static final ActionPathType LIST_METADATA_FORMATS = new ActionPathType("/listMetadataFormats");
  
  public static final ActionPathType MODIFY_AGENT = new ActionPathType("/modifyAgent");
  public static final ActionPathType MODIFY_AGGREGATOR = new ActionPathType("/modifyAggregator");
  public static final ActionPathType MODIFY_METADATA = new ActionPathType("/modifyMetadata");
  public static final ActionPathType MODIFY_METADATA_PROVIDER = new ActionPathType("/modifyMetadataProvider");
  public static final ActionPathType MODIFY_RESOURCE = new ActionPathType("/modifyResource");

  public static final ActionPathType TRANSLATE = new ActionPathType("/translate");
  
  // NOTE:  enum values must exactly match those above
  /** enum provided for switch and for statements */
  public static enum ActionPath {
  	ADD_AGENT,
    ADD_AGGREGATOR,
    ADD_METADATA,
    ADD_METADATA_PROVIDER,
    ADD_RESOURCE,
    COUNT_MEMBERS,
    DESCRIBE,
    FIND,
    FIND_AGENT,
    FIND_METADATA,
    FIND_RESOURCE,
    GET,
    GET_CONTENT,
    GET_METADATA,
    GET_METADATA_ABOUT,
    LIST_MEMBERS,
    LIST_METADATA_FORMATS,
    MODIFY_AGENT,
    MODIFY_AGGREGATOR,
    MODIFY_METADATA,
    MODIFY_METADATA_PROVIDER,
    MODIFY_RESOURCE,
    TRANSLATE,
  }
  
}
