/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth;

import java.util.Set;

import org.apache.log4j.Logger;
import org.nsdl.repository.api.request.Request;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;

public class MockAgentAuthInfo implements AgentAuthInfo {

	private static final Logger logger = Logger
			.getLogger(MockAgentAuthInfo.class);
	private final NDRObject agent;
	private final boolean verified;
	private String authorizedObjectHandle;

	public MockAgentAuthInfo(NDRObject agent, boolean verified) {
		this.agent = agent;
		this.verified = verified;
	}

	public boolean isVerified() {
		return verified;
	}

	public NDRObject agent() {
		return this.agent;
	}

	public boolean memberOf(Set<NDRObject> s) {
		return false;
	}

	public boolean memberOf(NDRObjectInfo s) {
		return false;
	}

	public boolean isAuthorizedFor(NDRObjectInfo o) {
		logger.debug("Checking authorized agent:  authorized is "
				+ this.authorizedObjectHandle + ", given is: " + o.handle());
		return (o.handle().equals(this.authorizedObjectHandle));
	}

	public boolean isAuthorizedFor(Request r) {
		return false;
	}

	public String getMessage() {
		return null;
	}

	public void addMessage(String message) {

	}

	public void setAuthorizedFor(String handle) {
		this.authorizedObjectHandle = handle;
	}

}
