/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth;

import java.security.KeyException;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;

import org.junit.Before;
import org.junit.Test;
import org.nsdl.util.security.RSAtoXMLKeyTranslator;

import static junit.framework.Assert.assertEquals;

public class RSAtoXMLKeyTranslatorTest {
	private PrivateKey privateKey;
	private PublicKey publicKey;
	private RSAtoXMLKeyTranslator translator = new RSAtoXMLKeyTranslator();

	@Before
	public void setUp() throws NoSuchAlgorithmException {
		if (this.privateKey == null || this.publicKey == null) {
			KeyPairGenerator generator = KeyPairGenerator.getInstance("RSA");
			KeyPair pair = generator.generateKeyPair();
			this.privateKey = pair.getPrivate();
			this.publicKey = pair.getPublic();
		}
	}

	@Test
	public void testPublicKeyEquality() throws KeyException {
		String keySource = translator.encodePublicKey(publicKey);
		PublicKey readKey = translator.decodePublicKey(keySource);
		String readKeySource = translator.encodePublicKey(readKey);

		assertEquals(keySource, readKeySource);
	}

	@Test
	public void testPrivateKeyEquality() throws KeyException {
		String keySource = translator.encodePrivateKey(privateKey);
		PrivateKey readKey = translator.decodePrivateKey(keySource);
		String readKeySource = translator.encodePrivateKey(readKey);

		assertEquals(keySource, readKeySource);
	}

	/* TODO: Need to test passphrase wrapping, once that's inplemented */

	/* Need to create tests with multiple keys, if we're supporting that.. */

}
