/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.auth.rules;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;
import org.nsdl.repository.model.Relationship;
import org.nsdl.repository.model.content.CommonModel.Components;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.NDRObjectTemplate;
import org.nsdl.repository.util.StaticObjectInfo;

import static junit.framework.Assert.*;

public class MemberOfRelationshipsTest {
    private ComponentRule rule = new MemberOfRelationships();

    private static final List<Relationship> relationships =
            Arrays.asList(new Relationship[] {

                    new Relationship(new StaticObjectInfo("MemberResource",
                            Type.Resource),Components.MEMBER_OF,
                            new NDRObjectTemplate(Type.Aggregator,
                                    "AggregationHandle")),

                    new Relationship(new StaticObjectInfo("MemberAggregation",
                            Type.Aggregator), Components.MEMBER_OF,
                            new NDRObjectTemplate(Type.Aggregator,
                                    "AggregationHandle")),

                    new Relationship(new StaticObjectInfo("MemberAgent",
                            Type.Agent), Components.MEMBER_OF,
                            new NDRObjectTemplate(Type.Aggregator,
                                    "AggregationHandle")), });

    /*
     * public void testAcceptance() { AuthAgentTestStub authAgent = new
     * AuthAgentTestStub( new NDRObjectTemplate(NDRObjectInfo.Type.Agent,
     * "Authorized"), true);
     * 
     * authAgent.setAuthorizedFor("AggregationHandle");
     * 
     * Iterator<Relationship> i = relationships.iterator(); while(i.hasNext()) {
     * Relationship r = i.next(); assertTrue(rule.checkCompliance( r, new
     * TestRequestStub(r.subject(), "addAggregator"), authAgent)); } }
     */
    @Test
    public void testMatchingPattern() {

        for (Relationship r : relationships) {
            assertTrue(rule.matchingPatterns().contains(
                    Relationship.Pattern.getPattern(r)));
        }

        assertEquals(rule.matchingPatterns().size(), relationships.size());
    }
}
