/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.request;

import org.junit.Test;
import org.nsdl.repository.api.request.RequestUtils.ApiRequestPath;

import servletunit.HttpServletRequestSimulator;
import servletunit.ServletContextSimulator;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertTrue;

public class RequestUtilsTest {
    @Test
    public void pathTest_nohandle_nopath() {
        ApiRequestPath path = getRequestPath("/add");

        assertFalse(path.hasHandle());
        assertFalse(path.hasPath());
        assertEquals("add", path.getApiCall());
    }

    @Test
    public void pathTest_cnri_nopath() {
        ApiRequestPath path = getRequestPath("/modify/2200/TEST");

        assertTrue(path.hasHandle());
        assertFalse(path.hasPath());
        assertEquals("modify", path.getApiCall());
        assertEquals("2200/TEST", path.getHandle());
    }

    @Test
    public void pathTest_pid_nopath() {
        ApiRequestPath path = getRequestPath("/modify/test:TEST");

        assertTrue(path.hasHandle());
        assertFalse(path.hasPath());
        assertEquals("modify", path.getApiCall());
        assertEquals("test:TEST", path.getHandle());
    }

    @Test
    public void pathTest_cnri_path() {
        ApiRequestPath path = getRequestPath("/get/2200/TEST/DATASTREAM");

        assertTrue(path.hasHandle());
        assertTrue(path.hasPath());
        assertEquals("get", path.getApiCall());
        assertEquals("2200/TEST", path.getHandle());
        assertEquals(path.getPath().length, 1);
        assertEquals(path.getPath()[0], "DATASTREAM");
    }

    @Test
    public void pathTest_pid_path() {
        ApiRequestPath path = getRequestPath("/get/test:TEST/DATASTREAM");

        assertTrue(path.hasHandle());
        assertTrue(path.hasPath());
        assertEquals("get", path.getApiCall());
        assertEquals("test:TEST", path.getHandle());
        assertEquals(path.getPath().length, 1);
        assertEquals(path.getPath()[0], "DATASTREAM");
    }

    public ApiRequestPath getRequestPath(String path) {
        HttpServletRequestSimulator simulator = new HttpServletRequestSimulator(
                new ServletContextSimulator());
        simulator.setPathInfo(path);
        return RequestUtils.getPath(simulator);
    }
}
