/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.request.filters;

import org.junit.Test;
import org.nsdl.repository.api.request.MockRequest;
import org.nsdl.repository.api.request.Request.Commands;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.model.content.ResourceModel;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.StaticObjectInfo;

import static junit.framework.Assert.assertEquals;

public class URINormalizationFilterTest {

    /** Assure that the filter does actually normalize URLs */
    @Test
    public void doesNormalizationTest() {
        MockRequest rq = new MockRequest();
        rq.putProperty(Commands.MATCH, new Property(new StaticObjectInfo(
                "test", Type.Resource),
                ResourceModel.Components.IDENTIFIER_URL, "http://NSDL.ORG"));

        URINormalizationFilter filter = new URINormalizationFilter();
        filter.process(rq);

        String filteredURL = rq.getProperties(Commands.MATCH).iterator().next()
                .value();
        String expectedURL = "http://nsdl.org/";
        assertEquals(expectedURL, filteredURL);
    }

    /** Assure that filter doesn't barf when no URL */
    @Test
    public void noURLTest() {
        MockRequest rq = new MockRequest();
        URINormalizationFilter filter = new URINormalizationFilter();
        filter.process(rq);

        /* No exception should occur */
    }
}
