/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.api.util;

import java.text.ParseException;
import java.util.HashSet;

import org.nsdl.mptstore.rdf.ObjectNode;
import org.nsdl.mptstore.rdf.PredicateNode;
import org.nsdl.mptstore.rdf.SubjectNode;
import org.nsdl.mptstore.rdf.Triple;
import org.nsdl.mptstore.util.NTriplesUtil;
import org.nsdl.repository.impl.fedora.risearch.TestRIQueryEngine;

public class TripleTesterUtils {
    /** Add a triple to the mock RI.  We assume that the object is a literal */
    public static  void useTripleLiteral(String s, String p, String o, TestRIQueryEngine ri) 
            throws ParseException {
        if (!s.startsWith("<")) {
            s = "<" + s  + ">";
        }
        SubjectNode subject = NTriplesUtil.parseSubject(s);
        
        if (!p.startsWith("<")) {
            p = "<" + p + ">";
        }
        
        PredicateNode predicate = NTriplesUtil.parsePredicate(p);
        
        /* In this unit test, all objects are literals */
        if (!o.startsWith("\"")) {
            o = "\"" + o + "\"";
        }
        
        ObjectNode object = NTriplesUtil.parseObject(o);
        
        HashSet<Triple> triples = new HashSet<Triple>();
        triples.add(new Triple(subject, predicate, object));
        ri.populate(triples.iterator());
    }
}
