/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.impl.fedora;

import org.apache.commons.httpclient.HttpClient;
import org.nsdl.repository.impl.fedora.FedoraAPIWrapper;
import org.nsdl.repository.impl.fedora.FedoraAccess;
import org.nsdl.repository.impl.fedora.risearch.TestRIQueryEngine;

import fedora.common.PID;

/** Simple fedora access object with mutable API/RI */
public class MockFedoraAccess implements FedoraAccess {

    private FedoraAPIWrapper wrapper;
    private TestRIQueryEngine ri;

    /**
     * Set the API wrapper instance
     * 
     * @param api
     */
    public void setAPI(FedoraAPIWrapper api) {
        this.wrapper = api;
    }

    /**
     * Sets the RIQueryEngine instance
     * 
     * @param ri
     */
    public void setRI(TestRIQueryEngine ri) {
        this.ri = ri;
    }

    public FedoraAPIWrapper getAPI() {
        return wrapper;
    }

    public HttpClient getHttpClient() {
        return null;
    }

    public TestRIQueryEngine getRI() {
        return ri;
    }

    public boolean objectExists(PID pid) {
        throw new UnsupportedOperationException("Not implemented");
    }
}
