/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.impl.fedora;

import org.junit.BeforeClass;
import org.junit.Test;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.NDRObject.State;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.NDRObjectTemplate;

import static junit.framework.Assert.*;

public class MockFedoraNDRAccessTest {

    private static NDRAccess ndr;

    @BeforeClass
    public static void setUp() {
        ndr = MockFedoraNDRAccess.getInstance();
    }

    @Test
    public void stateTest() {

        NDRObject opened;

        NDRObject resource = ndr.newObject(Type.Resource);
        resource.commit();

        NDRObjectTemplate template = new NDRObjectTemplate(Type.Resource);
        resource.setHandle(resource.handle());
        Iterable<NDRObjectInfo> results;

        /* make sure object state claims to be active */
        assertTrue("Default state should be active", resource.state().equals(
                State.Active));

        /* Make sure object can be found to be active */
        template.setState(State.Active);
        results = ndr.getObjectFinder().matchObject(template);
        assertTrue("Should be able to find default active resource", results
                .iterator().hasNext());

        /* Set to inactive but DON"t commut */
        resource.setState(State.Inactive);
        template.setState(State.Inactive);
        assertTrue("Should be able to read back changed state", resource
                .state().equals(State.Inactive));
        results = ndr.getObjectFinder().matchObject(template);
        assertFalse("Uncommitted state change should not be findable", results
                .iterator().hasNext());
        opened = ndr.getObject(resource);
        assertTrue(
                "Opened resource should show repository state, not currently mutated state",
                opened.state().equals(State.Active));

        /* Commit resource with new Inactive state */
        resource.commit();
        assertTrue("Should be able to read back changed state", resource
                .state().equals(State.Inactive));
        results = ndr.getObjectFinder().matchObject(template);
        assertTrue("Cmmitted state change should be findable", results
                .iterator().hasNext());
        opened = ndr.getObject(resource);
        assertTrue("Opened resource should show committed state", opened
                .state().equals(State.Inactive));


        opened.setState(State.Deleted);
        opened.commit();
        template.setState(State.Deleted);
        results = ndr.getObjectFinder().matchObject(template);
        assertTrue("Committed state change should be findable", results
                .iterator().hasNext());
    }
}
