/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.impl.fedora.risearch;

import java.io.*;
import java.util.Iterator;

import org.nsdl.mptstore.rdf.Triple;
import org.nsdl.mptstore.util.NTriplesUtil;

/** Iterates triples found in an NTriple-formatted file, readaer, etc.
 * 
 * @author birkland
 *
 */
public class NTripleIterator implements Iterator<Triple> {
    private final BufferedReader reader;
    private String currentLine;
    
    public NTripleIterator(Reader tripleReader)
    	throws IOException
    {
    	reader = new BufferedReader(tripleReader);
    	currentLine = reader.readLine();
    }
    
    public NTripleIterator(File tripleFile) 
            throws FileNotFoundException, IOException {
        reader = new BufferedReader(new FileReader(tripleFile));
        currentLine = reader.readLine();
    }
    
    public boolean hasNext() {
        return currentLine != null;
    }
    
    public Triple next() {
        Triple t;
        
        try {
            t = NTriplesUtil.parseTriple(currentLine);
            currentLine = reader.readLine();
        } catch (Exception e) {
            throw new RuntimeException("Error reading trples", e);
        }
        return t;
    }
    
    public void remove()  {
        throw new UnsupportedOperationException("remove() not supported");
    }

}
