/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model;

import org.junit.Test;
import org.nsdl.repository.model.Property.Pattern;
import org.nsdl.repository.model.content.CommonModel.Components;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.StaticObjectInfo;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertTrue;

import static org.nsdl.repository.model.content.OAIModel.Set.Components.SET_SPEC;

public class PropertyTest {

    private Pattern baselinePattern = new Pattern(Type.MetadataProvider,
            SET_SPEC);

    private Property baselineProperty = new Property(new StaticObjectInfo(
            "test", Type.MetadataProvider), SET_SPEC, "testSpec");

    @Test
    public void equalityTest() {

        assertEquals(baselinePattern, new Pattern(Type.MetadataProvider,
                SET_SPEC));
        assertEquals(baselinePattern, Pattern.getPattern(baselineProperty));
    }

    @Test
    public void inequalityTest() {
        assertFalse(baselinePattern
                .equals(new Pattern(Type.Metadata, SET_SPEC)));
        assertFalse(baselinePattern.equals(new Pattern(Type.MetadataProvider,
                Components.HAS_HANDLE)));
    }

    @Test
    public void patternMatchTest() {
        Property toMatch = new Property(new StaticObjectInfo("irrelevant",
                Type.MetadataProvider), SET_SPEC, "irrelevant");

        assertTrue(baselinePattern.matches(toMatch));
    }

    @Test
    public void patternMismatchTest() {
        Property noMatch1 = new Property(new StaticObjectInfo("irrelevant",
                Type.Metadata), SET_SPEC, "irrelevant");
        Property noMatch2 = new Property(new StaticObjectInfo("irrelevant",
                Type.MetadataProvider), Components.HAS_HANDLE, "irrelevant");

        assertFalse(baselinePattern.matches(noMatch1));
        assertFalse(baselinePattern.matches(noMatch2));
    }
}
