/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model;

import org.junit.Test;
import org.nsdl.repository.model.Relationship.Pattern;
import org.nsdl.repository.model.content.CommonModel.Components;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.NDRObjectTemplate;
import org.nsdl.repository.util.StaticObjectInfo;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertTrue;

import static org.nsdl.repository.model.content.MetadataProviderModel.Components.METADTA_PROVIDER_FOR;

public class RelationshipTest {
    private Pattern baselinePattern = new Pattern(Type.MetadataProvider,
            METADTA_PROVIDER_FOR, Type.Agent);
    private Relationship baselineRelationship = new Relationship(
            new StaticObjectInfo("baseMP", Type.MetadataProvider),
            METADTA_PROVIDER_FOR,
            new NDRObjectTemplate(Type.Agent, "baseAgemt"));

    @Test
    public void equalityTest() {
        assertEquals(baselinePattern, new Pattern(Type.MetadataProvider,
                METADTA_PROVIDER_FOR, Type.Agent));
        assertEquals(baselinePattern, Pattern.getPattern(baselineRelationship));
    }

    @Test
    public void inequalityTest() {
        assertFalse(baselinePattern.equals(new Pattern(Type.Aggregator,
                METADTA_PROVIDER_FOR, Type.Agent)));
        assertFalse(baselinePattern.equals(new Pattern(Type.MetadataProvider,
                METADTA_PROVIDER_FOR, Type.Resource)));
        assertFalse(baselinePattern.equals(new Pattern(Type.MetadataProvider,
                Components.MEMBER_OF, Type.Agent)));
    }

    @Test
    public void patternMatchTest() {
        Relationship toMatch = new Relationship(new StaticObjectInfo(
                "irrelevant", Type.MetadataProvider), METADTA_PROVIDER_FOR,
                new NDRObjectTemplate(Type.Agent, "baseAgemt"));
        assertTrue(baselinePattern.matches(toMatch));
    }

    @Test
    public void patternMismatchTest() {
        Relationship noMatch1 = new Relationship(new StaticObjectInfo(
                "irrelevant", Type.Aggregator), METADTA_PROVIDER_FOR,
                new NDRObjectTemplate(Type.Agent, "irrelevant"));

        Relationship noMatch2 = new Relationship(new StaticObjectInfo(
                "irrelevant", Type.MetadataProvider), METADTA_PROVIDER_FOR,
                new NDRObjectTemplate(Type.Resource, "irrelevant"));

        Relationship noMatch3 = new Relationship(new StaticObjectInfo(
                "irrelevant", Type.MetadataProvider), Components.MEMBER_OF,
                new NDRObjectTemplate(Type.Agent, "irrelevant"));

        assertFalse(baselinePattern.matches(noMatch1));
        assertFalse(baselinePattern.matches(noMatch2));
        assertFalse(baselinePattern.matches(noMatch3));
    }
}
