/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.content;

import org.junit.BeforeClass;
import org.junit.Test;
import org.nsdl.repository.access.NDRAccess;
import org.nsdl.repository.impl.fedora.MockFedoraNDRAccess;
import org.nsdl.repository.model.content.CommonModel.Components;
import org.nsdl.repository.model.types.Type;
import org.nsdl.repository.util.NDRObjectTemplate;

public class CommonModelTest {

    private static NDRAccess ndr;
    private static CommonModel model;

    @BeforeClass
    public static void setup() {
        ndr = MockFedoraNDRAccess.getInstance();
        model = new CommonModel();
        model.init(ndr);
    }

    @Test
    public void basicAcceptanceTest() throws ContentModelException {
        NDRObjectTemplate template = new NDRObjectTemplate(Type.Metadata);
        template.setHandle("2200/Test");
        template.addProperty(Components.OBJECT_TYPE, Type.Metadata.toString());

        model.checkCompliance(template);
    }

    @Test(expected = ContentModelException.class)
    public void tooManyHandlesTest() throws ContentModelException {
        NDRObjectTemplate template = new NDRObjectTemplate(Type.Metadata);
        template.addProperty(Components.HAS_HANDLE, "2200/test1");
        template.addProperty(Components.HAS_HANDLE, "2200/test2");
        template.addProperty(Components.OBJECT_TYPE, Type.Metadata.toString());

        model.checkCompliance(template);
    }

    @Test(expected = ContentModelException.class)
    public void notEnoughHandlesTest() throws ContentModelException {
        NDRObjectTemplate template = new NDRObjectTemplate(Type.Metadata);
        template.addProperty(Components.OBJECT_TYPE, Type.Metadata.toString());

        model.checkCompliance(template);
    }

    @Test(expected = ContentModelException.class)
    public void noObjectTypesTest() throws ContentModelException {
        NDRObjectTemplate template = new NDRObjectTemplate(Type.Metadata);
        template.removeProperties(template
                .getProperties(Components.OBJECT_TYPE));
        template.addProperty(Components.HAS_HANDLE, "2200/test");

        model.checkCompliance(template);
    }

    @Test
    public void moreThanOneObjectTypeTest() throws ContentModelException {
        NDRObjectTemplate template = new NDRObjectTemplate(Type.Metadata);
        template.setHandle("2200/test");
        template.addProperty(Components.OBJECT_TYPE, Type.Resource.toString());
        template.addProperty(Components.OBJECT_TYPE, Type.Agent.toString());

        model.checkCompliance(template);
    }

    @Test(expected = ContentModelException.class)
    public void badObjectTypeTest() throws ContentModelException {
        NDRObjectTemplate template = new NDRObjectTemplate(Type.Metadata);
        template.addProperty(Components.HAS_HANDLE, "2200/test");
        template.addProperty(Components.OBJECT_TYPE, "does_not_exist");

        model.checkCompliance(template);
    }
}
