/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.impl;

import java.util.Arrays;
import java.util.Collection;

import org.junit.Test;
import org.nsdl.repository.model.NDRObject;
import org.nsdl.repository.model.NDRObjectInfo;
import org.nsdl.repository.model.Property;
import org.nsdl.repository.util.NDRObjectTemplate;
import org.nsdl.repository.util.StaticObjectInfo;

import static junit.framework.Assert.*;

public class PropertyTrackerTest {

    private static final Property.Name q1 = new Property.Name("info/test#1",
            "test_1", "t");
    private static final Property.Name q2 = new Property.Name("info/test#2",
            "test_2", "t");
    private static final Property.Name q3 = new Property.Name("info/test#3",
            "test_3", "t");

    private static final NDRObjectInfo o1 = new StaticObjectInfo("Test1");
    // private static final NDRObjectInfo o2 = new StaticObjectInfo("Test2");
    // private static final NDRObjectInfo o3 = new StaticObjectInfo("Test3");

    private static final Property ps1 = new Property(o1, q1, "test1");
    private static final Property ps2 = new Property(o1, q2, "test2");
    private static final Property ps3 = new Property(o1, q2, "test3");
    private static Collection<Property> propertySample = Arrays.asList(ps1,
            ps2, ps3);

    @Test
    public void testSimpleAdd() {
        BasicComponentTracker<Property> tracker = getPopulatedTracker();

        Property added = new Property(o1, q3, "testSA");
        tracker.add(added);

        assertTrue(tracker.hasModified());
        assertEquals(added, tracker.getAdded().iterator().next());
    }

    @Test
    public void testSimpleDelete() {
        BasicComponentTracker<Property> tracker = getPopulatedTracker();

        tracker.remove(ps2);
        tracker.remove(ps3);

        assertTrue(tracker.hasModified());
        assertEquals(1, tracker.getCurrent().size());
        assertEquals(2, tracker.getDeleted().size());
        assertFalse(tracker.getCurrent().contains(ps2));
        assertFalse(tracker.getCurrent().contains(ps3));
        assertTrue(tracker.getDeleted().contains(ps2));
        assertTrue(tracker.getDeleted().contains(ps3));
    }

    @Test
    public void testSimpleReplace() {
        BasicComponentTracker<Property> tracker = getPopulatedTracker();
        Property p = tracker.get(q1).iterator().next();
        p.setValue("replaceValue");

        assertTrue("Tracker did not detect update", tracker.hasModified());
        assertEquals(tracker.getCurrent().size(), propertySample.size());
        assertTrue(tracker.getUpdated().contains(ps1));
    }

    @Test
    public void testDuplicateEqualitySkip() {
        BasicComponentTracker<Property> tracker = getPopulatedTracker();

        tracker.add(ps1);

        assertFalse(tracker.hasModified());
        assertEquals(tracker.getAdded().size(), 0);
    }

    public static PropertyTracker getPopulatedTracker() {
        NDRObject base = new NDRObjectTemplate(null);
        base.addProperties(propertySample);
        return new PropertyTracker(base);
    }
}
