/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.types;

import org.junit.BeforeClass;
import org.junit.Test;
import org.nsdl.repository.impl.fedora.MockFedoraNDRAccess;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertNotNull;

public class AgentTests {
    private static MockFedoraNDRAccess ndr;

    @BeforeClass
    public static void setupNDR() {
        ndr = MockFedoraNDRAccess.getInstance();
    }

    @Test
    public void notNullTitleTest() {
        Agent agent = ndr.newObject(Type.Agent);
        assertNotNull(agent.getTitle());
    }

    @Test
    public void setTitleTest() {
        String TITLE = "title";
        Agent agent = ndr.newObject(Type.Agent);
        agent.setTitle(TITLE);
        agent.commit();
        assertEquals(TITLE, agent.getTitle());
    }

    @Test
    public void createAggregatorsTest() {
        Agent agent = ndr.newObject(Type.Agent);
        agent.commit();

        Aggregator agg1 = agent.newAggregator();
        agg1.commit();
        Aggregator agg2 = agent.newAggregator();
        agg2.commit();

        assertEquals(2, agent.getAggregators().size());
    }

    @Test
    public void createMetadataProvidersTest() {
        Agent agent = ndr.newObject(Type.Agent);
        agent.commit();

        Aggregator agg1 = agent.newAggregator();
        agg1.commit();
        Aggregator agg2 = agent.newAggregator();
        agg2.commit();

        assertEquals(2, agent.getAggregators().size());
    }
}
