/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.model.types;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.junit.Test;
import org.nsdl.repository.model.types.Type.UnknownTypeException;

import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertTrue;

public class TypeTest {
    @Test
    public void correctTypeNamesTest() {
        String message = "Type name did not match";
        assertEquals(message, "Agent", Type.Agent.toString());
        assertEquals(message, "Aggregator", Type.Aggregator.toString());
        assertEquals(message, "Metadata", Type.Metadata.toString());
        assertEquals(message, "MetadataProvider", Type.MetadataProvider
                .toString());
        assertEquals(message, "Resource", Type.Resource.toString());
    }

    @Test
    public void correctInstanceTest() {
        String message = "Type instantiation is wrong";
        assertEquals(message, Type.Agent, Type.getInstance("Agent"));
        assertEquals(message, Type.Aggregator, Type.getInstance("Aggregator"));
        assertEquals(message, Type.Metadata, Type.getInstance("Metadata"));
        assertEquals(message, Type.MetadataProvider, Type
                .getInstance("MetadataProvider"));
        assertEquals(message, Type.Resource, Type.getInstance("Resource"));
    }

    @SuppressWarnings("unchecked")
    @Test
    public void existingTypesTest() {
        Set<Type> existing = new HashSet<Type>(Arrays.asList(Type.Agent,
                Type.Aggregator, Type.Metadata, Type.MetadataProvider,
                Type.Resource));
        assertTrue(existing.containsAll(Type.values()));
        assertEquals(existing.size(), Type.values().size());
    }

    @Test(expected = UnknownTypeException.class)
    public void illegalTypeTest() {
        Type.getInstance("BAD");
    }
}
