/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.repository.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import org.junit.Test;

import static junit.framework.Assert.*;

public class ConfigurationTest {

    @Test
    public void singleProvidedFileTest() throws Exception {
        String contents = "property.findme = found";
        File propertyFile = createNewFile(contents, null);

        Configuration config = new Configuration(propertyFile);

        assertEquals("found", config.getProperty("property.findme"));
    }

    @Test
    public void multipleFilesTest() throws Exception {
        String contents1 = "property.findme.1 = found1";
        String contents2 = "property.findme.2 = found2";
        
        File configDir = mkdir();
        createNewFile(contents1, configDir);
        createNewFile(contents2, configDir);
        
        Configuration config = new Configuration(configDir);
        
        assertEquals("found1", config.getProperty("property.findme.1"));
        assertEquals("found2", config.getProperty("property.findme.2"));
        
    }

    private File createNewFile(String contents, File inDirectory)
            throws Exception {
        File testFile =
                File.createTempFile(ConfigurationTest.class.getSimpleName(),
                        ".test", inDirectory);
        testFile.deleteOnExit();
        writeToFile(contents, testFile);
        return testFile;
    }

    private void writeToFile(String contents, File fileToWriteTo)
            throws Exception {
        FileOutputStream out = new FileOutputStream(fileToWriteTo, false);
        out.write(contents.getBytes());
        out.close();
    }

    private File mkdir() throws Exception {
        File testDir =
                File.createTempFile(ConfigurationTest.class.getSimpleName(),
                        "_DIR");
        testDir.deleteOnExit();
        if (!testDir.delete()) throw new IOException();
        if (!testDir.mkdir()) throw new IOException();
        return testDir;
    }
}
