/*
 * ----------------------------------------------------------------------------
 *
 * License and Copyright:
 *
 * The contents of this file are subject to the Educational Community
 * License v1.0  (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 * http://www.opensource.org/licenses/ecl1.php.
 *
 * Software distributed under the License is distributed on an "AS IS" 
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * Copyright 2002-2009 by Cornell University All rights reserved.
 *
 * ----------------------------------------------------------------------------
 */
 
package org.nsdl.util.uri;

import junit.framework.TestCase;

import org.nsdl.util.uri.NormalizedURL;


/**
 * @author Edwin Shin
 */
public class TestNormalizedURL extends TestCase {

    public static final String nu1 = "http://www.example.org/";
    public static final String nu2 = "https://www.example.org/foo/bar/";
    public static final String nu3 = "ftp://example.org/%3Auser";
    public static final String nu4 = "http://example.com/X/y/Z/";
    public static final String nu5 = "http://example.com/a";
    public static final String nu6 = "http://example.com/~a?foo=bar#baz";
    public static final String nu7 = "http://example.com/%C3%87";
    public static final String nu8 = "file:///C:/Program%20Files";
    
    public static final String[] nu1_test1 = {
        "http://www.example.org/",
        "HTTP://WWW.EXAMPLE.ORG/",
        };
    
    public static final String[] nu1_test2 = {
            "http://www.example.org/",
            "http://www.example.org"
            };
    
    public static final String[] nu1_test3 = {
            "http://www.example.org:80/",
            "http://www.example.org:80",
            "http://www.example.org:/"
            };
    
    public static final String[] nu2_test2 = {
            "https://www.example.org/foo/../bar/../foo/bar/../../foo/../foo/bar/",
            "https://www.example.org/foo/bar/./",
            "https://www.example.org/foo/./bar/"
            };
    
    public static final String[] nu3_test1 = {
            "fTp://eXample.org/%3Auser",
            "ftp://example.org/%3auser"
            };
    
    public static final String[] nu4_test1 = {
            "http://example.com/X/y/Z/",
            "http://example.com/X/y/%5A/",
            "http://example.com/X/y/%5a/"
            };
    
    public static final String[] nu4_test2 = {
            "http://example.com/a%2Fb/c/%7A"
            };
    
    public static final String[] nu5_test1 = {
            "http://example.com/%61"
            };
    
    public static final String[] nu6_test1 = {
            "http://example.com/~a?foo=bar#baz",
            "http://example.com/%7Ea?foo=bar#baz"
            };
    
    public static final String[] nu7_test1 = {
            "http://example.com/%C3%87",
            "http://example.com/C%CC%A7"
            };
    
    public static final String[] nu8_test1 = {
            nu8
            };
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestNormalizedURL.class);
    }
    
    public void testCaseNormalization() throws Exception {
        NormalizedURL nurl;
        for (int i = 0; i < nu1_test1.length; i++) {
            nurl = new NormalizedURL(nu1_test1[i]);
            nurl.normalizeCase();
            assertEquals(nu1, nurl.toString());
        }
        
        for (int i = 0; i < nu3_test1.length; i++) {
            nurl = new NormalizedURL(nu3_test1[i]);
            nurl.normalizeCase();
            assertEquals(nu3, nurl.toString());
        }
    }
    
    public void testPercentEncodingNormalization() throws Exception {
        NormalizedURL nurl;
        for (int i = 0; i < nu4_test1.length; i++) {
            nurl = new NormalizedURL(nu4_test1[i]);
            nurl.normalizePercentEncoding();
            assertEquals(nu4, nurl.toString());
        }
        
        for (int i = 0; i < nu5_test1.length; i++) {
            nurl = new NormalizedURL(nu5_test1[i]);
            nurl.normalizePercentEncoding();
            assertEquals(nu5, nurl.toString());
        }
        
        for (int i = 0; i < nu6_test1.length; i++) {
            nurl = new NormalizedURL(nu6_test1[i]);
            nurl.normalizePercentEncoding();
            assertEquals(nu6, nurl.toString());
        }
        
        for (int i = 0; i < nu7_test1.length; i++) {
            nurl = new NormalizedURL(nu7_test1[i]);
            nurl.normalizePercentEncoding();
            assertEquals(nu7, nurl.toString());
        }
        
        for (int i = 0; i < nu8_test1.length; i++) {
            nurl = new NormalizedURL(nu8_test1[i]);
            nurl.normalizePercentEncoding();
            assertEquals(nu8, nurl.toString());
        }
    }
    
    public void testPathSegmentNormalization() throws Exception {
        NormalizedURL nurl;
        for (int i = 0; i < nu2_test2.length; i++) {
            nurl = new NormalizedURL(nu2_test2[i]);
            nurl.normalizePathSegment();
            assertEquals(nu2, nurl.toString());
        }
    }
    
    public void testSchemeBasedNormalization() throws Exception {
        NormalizedURL nurl;
        for (int i = 0; i < nu1_test2.length; i++) {
            nurl = new NormalizedURL(nu1_test2[i]);
            nurl.normalizeByScheme();
            assertEquals(nu1, nurl.toString());
        }
        for (int i = 0; i < nu1_test3.length; i++) {
            nurl = new NormalizedURL(nu1_test3[i]);
            nurl.normalizeByScheme();
            assertEquals(nu1, nurl.toString());
        }
    }
}
