/*
 * National Science Digital Library - Core Integration
 * Cornell University Information Science
 * 
 * Created on Oct 10, 2006
 */
package org.nsdl.repository.api;

import org.nsdl.repository.api.action.*;
import org.nsdl.repository.api.actionForm.Data;

import org.nsdl.util.date.*;

import servletunit.struts.CactusStrutsTestCase;

import java.util.Date;


/**
 * @author Naomi Dushay
 */
public abstract class BaseCactusTestCase extends CactusStrutsTestCase
{		
	/** this is to hold the request's path info and must be set by subclass */
	protected ActionPathType actionRequestPath;

	protected static final String ME = "BaseCactusTestCase";

	
	// TODO: set up the triple store in the correct place once per test class
	static
	{
	}

	/** constructor present to create TestSuite with specificly ordered tests */
	public BaseCactusTestCase(String testname)
	{
		super(testname);
	}

	
	//---------------------------------------------------------- protected methods

/*
	protected void addParamToRequest(String name, String value)
	{
		this.request.setAttribute(name, value);
		addRequestParameter(name, value);
	}
	
	protected void performStrutsAction(ActionPathType path, String successForwardStr)
	{
//		setRequestPathInfo(path.toString());
		actionPerform();
		processRequest(true);
	}
*/	
	
	/** perform a struts action that uses a Data form bean; validate response 
	 * properties common to all such actions. 
	 * @param actionRequestPath - the request's path
	 * @param successForwardStr - the literal used for forward for successful
	 * execution of the action */
	protected Data getDataResponse(String actionRequestPath, String successForwardStr)
	{		
		performAction(actionRequestPath);
		verifyForward(successForwardStr);

		return getBasicDataRespone(actionRequestPath);
	}

	/** perform a struts action and validate response data common to all actions. 
	 * @param actionRequestPath - the request's path
	 * @param successForwardStr - the literal used for forward for successful
	 * execution of the action */
	protected void assertErrorResponse(String actionRequestPath, String[] expectedActionErrs, String[] expectedActionMsgs)
	{		
		performAction(actionRequestPath);
		verifyForward(GenericAction.ERROR_FORWARD);
		verifyActionErrors(expectedActionErrs);		
		verifyActionMessages(expectedActionMsgs);
	}

	//----------------------------------------------------------- private methods

	/** sets the action's request path and requestURL, then performs the action
	 * @param actionRequestPath - the request's path
	 */
	private void performAction(String actionRequestPath)
	{
		setRequestPathInfo(actionRequestPath);
		actionPerform();
	}

	
	private Data getBasicDataRespone(String actionRequestPath)
	{
		Data dataBean = (Data) getActionForm();

		// check for errors
		verifyNoActionErrors();
		verifyNoActionMessages();		
		assertNull("Had error: " + dataBean.getCode(), dataBean.getCode());		

		// check requestUrl string
		String reqUrlStr = dataBean.getRequestURLStr();
		assertNotNull("Null requestURLStr", reqUrlStr);
		assertTrue("returned request string must end with " + actionRequestPath, reqUrlStr.endsWith(actionRequestPath));

		// check responseTime string
		String responseTimeStr = dataBean.getResponseTimeStr();
		assertNotNull("Null responseTimeStr", responseTimeStr);
		try
		{
			Date docDate = DateUtility.getDateFromISO8601Plus(responseTimeStr);
			// TODO: check if date is reasonable value
		}
		catch (DateUtilityException e)
		{
			fail("responseTime is not in correct format: " + e.getMessage());
		}
				
		return dataBean;
	}
		
}
