/*
 * National Science Digital Library - Core Integration
 * Cornell University Information Science
 * 
 * Created on Dec 4, 2006
 */
package org.nsdl.repository.api;

import org.nsdl.util.XMLTestUtil;
import org.xml.sax.SAXException;

import java.io.IOException;
import junit.framework.TestCase;

/**
 * tests for the XML schema for the inputXML argument for NDR requests
 * 
 * Given the complexity of the XML schema language, and the complexity of what 
 *  we're trying to describe with XML schemas for the inputXML, it's good to 
 *  have some tests to ensure our schema for inputXML is what we desire.
 * 
 *  NOTE:  this class requires the version of XMLUtil in the nsdlutil project,
 *  *NOT* the version of XMLutil in the NDR project.  It's a matter of configuring
 *  the classpath for this test ...
 *  
 *  and here is another "gotcha" for having multiple versions of the same class!!!
 * 
 * @author Naomi Dushay
 */
public class RequestXMLSchemaTest extends TestCase
{
//NOTE:  this class requires the version of XMLUtil in the nsdlutil project,
//*NOT* the version of XMLutil in this project.  It's a matter of configuring
//the classpath for this test ...
//
//and here is another "gotcha" for having multiple versions of the same class!!!


	private final static String OPEN_INPUT_XML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
	"<inputXML xmlns=\"http://ns.nsdl.org/ndr/request_v1.00/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" " +
	"xsi:schemaLocation=\"http://ns.nsdl.org/ndr/request_v1.00/ http://ns.nsdl.org/schemas/ndr/request_v1.00.xsd\" schemaVersion=\"1.00.000\">";

	private final static String CLOSE_INPUT_XML = "</inputXML>";

	private final static String OPEN_AGENT_INPUT_XML = OPEN_INPUT_XML + "<agent>";
	private final static String CLOSE_AGENT_INPUT_XML = "</agent>" + CLOSE_INPUT_XML;
	
	private final static String OPEN_AGGR_INPUT_XML = OPEN_INPUT_XML + "<aggregator>";
	private final static String CLOSE_AGGR_INPUT_XML = "</aggregator>" + CLOSE_INPUT_XML;
	
	private final static String OPEN_MD_INPUT_XML = OPEN_INPUT_XML + "<metadata>";
	private final static String CLOSE_MD_INPUT_XML = "</metadata>" + CLOSE_INPUT_XML;
	
	private final static String OPEN_MDP_INPUT_XML = OPEN_INPUT_XML + "<metadataProvider>";
	private final static String CLOSE_MDP_INPUT_XML = "</metadataProvider>" + CLOSE_INPUT_XML;
	
	private final static String OPEN_RESOURCE_INPUT_XML = OPEN_INPUT_XML + "<resource>";
	private final static String CLOSE_RESOURCE_INPUT_XML = "</resource>" + CLOSE_INPUT_XML;

	
	private final static String OPEN_DATA = "<data>";
	private final static String CLOSE_DATA = "</data>";
	private final static String OPEN_PROPS = "<properties>";
	private final static String CLOSE_PROPS = "</properties>";
	private final static String OPEN_RELS = "<relationships>";
	private final static String CLOSE_RELS = "</relationships>";

	private final static String EMPTY_DATA = OPEN_DATA + CLOSE_DATA;
	private final static String EMPTY_PROPS = OPEN_PROPS + CLOSE_PROPS;
	
	private final static String MD_PROPS = OPEN_PROPS 
		+	"<uniqueID>i.can.be.any.xml.schema.token</uniqueID>"
		+ "<itemId>oai:nsdl.org:NSDL/666</itemId>"
		+ CLOSE_PROPS;
	
	private final static String MDP_PROPS = OPEN_PROPS 
		+	"<setSpec>test_set_spec</setSpec>"
		+ "<setName>\"Pretty name\" for set goes here</setName>"
		+ CLOSE_PROPS;

	private final static String RES_URL_ID_PROPS = OPEN_PROPS 
		+	"<identifier type=\"URL\">http://www.stuffonmycat.com</identifier>"
		+ CLOSE_PROPS;

	private final static String RES_HOST_ID_PROPS = OPEN_PROPS 
		+	"<identifier type=\"HOST\">www.stuffonmycat.com</identifier>"
		+ CLOSE_PROPS;

	
	private final static String AGGR_RELS = OPEN_RELS 
		+ "<associatedWith>2200/20061002130909213T</associatedWith>"
		+ "<aggregatorFor>2200/20061002125006088T</aggregatorFor>"
		+ CLOSE_RELS;

	private final static String MEMBER_OF_REL = OPEN_RELS 
		+ "<memberOf>2200/NSDL_Collection_of_Resources</memberOf>"
		+ CLOSE_RELS;

	private final static String MD_RELS = OPEN_RELS 
		+	"<metadataFor>2200/20061002172537014T</metadataFor>"
		+ "<metadataProvidedBy>2200/20061002125005203T</metadataProvidedBy>"
		+ CLOSE_RELS;

	private final static String MDP_RELS = OPEN_RELS 
		+ "<aggregatedBy>2200/20061002130909950T</aggregatedBy>"
		+ "<metadataProviderFor>2200/20061002125006088T</metadataProviderFor>"
		+ CLOSE_RELS;

	private final static String FEDORA_DC_DATA = OPEN_DATA + "<DC>"
		+ "<oai_dc:dc xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:oai_dc=\"http://www.openarchives.org/OAI/2.0/oai_dc/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd\">"
		+ "<dc:title>www.mathdl.org Agent</dc:title>"
		+ "</oai_dc:dc>" + "</DC>"	+ CLOSE_DATA;
	
	private final static String SAMPLE_IMAGE = "<image>"
		+ "<brandURL>http://crs.nsdl.org/brands/1523829.jpg</brandURL>"
		+ "<title>MathDL: Digital Classroom Resources</title>"
		+ "<width>100</width>"
		+ "<height>30</height>"
		+ "</image>";

	private final static String SAMPLE_CONTACT = "<contact>"
		+ "<name>I.M.A. Geek</name>" 
		+ "<email>geek@nsdl.org</email>" 
		+ "<info>Could be any one of us!</info>" 
		+ "</contact>";
		
	private final static String HARVEST_INFO_DATA = OPEN_DATA
		+ "<harvestInfo xmlns=\"http://ns.nsdl.org/MRingest/harvest_v1.00/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://ns.nsdl.org/MRingest/harvest_v1.00/ http://ns.nsdl.org/schemas/ndr/ndr_ingest.xsd\">"
			+ "<harvestRequest schemaVersion=\"1.00.000\">"
				+ "<baseURL>http://mathdl.maa.org/oai2/oai</baseURL>"
				+ "<collectionNA>1523829</collectionNA>"
				+ "<runType>harvest</runType>"
				+ "<providerEmail>mr-ingest@nsdl.org</providerEmail>"
				+ "<requestedChecks/>"
				+ "<fromDate>2006-11-21</fromDate>"
				+ "<sets>"
					+ "<set>aSetSpec</set>"
				+ "</sets>"
				+ "<formats>"
					+ "<format>anOAImetadataPrefix</format>"
				+ "</formats>"
				+ "<firstHarvest>true</firstHarvest>"
				+ "<uuid/>"
			+ "</harvestRequest>"
		+ "</harvestInfo>" + CLOSE_DATA;
	
	private final static String SERVICE_DESC_DATA = OPEN_DATA
	+ "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:title>title of Aggregation</dc:title>"
		+ "<dc:description>description of Aggregation</dc:description>" 
		+ "<dc:identifier>http://somewhere.over.rainbow</dc:identifier>"
		+ SAMPLE_IMAGE
		+ "<contacts>" 
			+ SAMPLE_CONTACT
		+ "</contacts>"
	+ "</serviceDescription>"	+ CLOSE_DATA;
	
	private final static String FORMAT_DATA = OPEN_DATA	+ "<format type=\"nsdl_dc\">"
	+ "<meta>"
		+ "<nsdl_dc:nsdl_dc xmlns:nsdl_dc=\"http://ns.nsdl.org/nsdl_dc_v1.02/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:dct=\"http://purl.org/dc/terms/\" xmlns:ieee=\"http://www.ieee.org/xsd/LOMv1p0\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" schemaVersion=\"1.02.000\" xsi:schemaLocation=\"http://ns.nsdl.org/nsdl_dc_v1.02/ http://ns.nsdl.org/schemas/nsdl_dc/nsdl_dc_v1.02.xsd\">"
			+ "<dc:title>sample title</dc:title>"
			+ "<dc:subject>sample subject</dc:subject>"
		+ "</nsdl_dc:nsdl_dc>"
	+ "</meta>"
	+ "<info>"
		+ "<nsdlAboutCategory>collection</nsdlAboutCategory>"
		+ "<repositoryPrimaryIdentifier>http://www.mathdl.org/mathDL/3/</repositoryPrimaryIdentifier>"
		+ "<link>oai:nsdl.org:nsdl:3000:nsdl:4002</link>"
		+ "<harvestDate>2006-12-06T16:11:30Z</harvestDate>"
		+ "<harvestHarvestType>OAI2.0</harvestHarvestType>"
		+ "<harvestDatasourcePublic>true</harvestDatasourcePublic>"
		+ "<harvestDatasource>http://crs.nsdl.org/oai2.php</harvestDatasource>"
		+ "<harvestIdentifier>oai:crs.nsdl.org:1523829</harvestIdentifier>"
		+ "<harvestRecordDatestamp>2006-02-01T16:11:30Z</harvestRecordDatestamp>"
		+ "<metadataNamespace>http://ns.nsdl.org/nsdl_dc_v1.02/</metadataNamespace>"
	+ "</info>"
	+ "</format>"	+ CLOSE_DATA;

	/** Ensures the <data>, <properties>, <relationships> elements are not required */
	public void testElementsAreOptional()
		throws SAXException, IOException
	{
		// all are missing
		String inputXML = OPEN_AGENT_INPUT_XML + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
	}

	
	/** Ensures the order of data/properties/relationships elements doesn't matter */
	public void testOrderDoesNotMatter()
		throws SAXException, IOException
	{
		// agent - fedoraDCdatastream, resourceIdentifier props, memberOf rels
		assertAllOrderingValidates(OPEN_AGENT_INPUT_XML, FEDORA_DC_DATA, RES_HOST_ID_PROPS, MEMBER_OF_REL, CLOSE_AGENT_INPUT_XML);

		// aggregator - serviceDescription datastream, no props, aggregator relationships
		assertAllOrderingValidates(OPEN_AGGR_INPUT_XML, SERVICE_DESC_DATA, EMPTY_PROPS, AGGR_RELS, CLOSE_AGGR_INPUT_XML);
		
		// metadata - format datastream, metadata properties and relationships
		assertAllOrderingValidates(OPEN_MD_INPUT_XML, FORMAT_DATA, MD_PROPS, MD_RELS, CLOSE_MD_INPUT_XML);
		
		// metadataProvider - serviceDescription datastream, metadataProvider properties and relationships
		assertAllOrderingValidates(OPEN_MDP_INPUT_XML, HARVEST_INFO_DATA, MDP_PROPS, MDP_RELS, CLOSE_MDP_INPUT_XML);
		
		// resource - no datastream, resourceIdentifier props, memberOf rels
		assertAllOrderingValidates(OPEN_RESOURCE_INPUT_XML, EMPTY_DATA, RES_URL_ID_PROPS, MEMBER_OF_REL, CLOSE_RESOURCE_INPUT_XML);		
	}
	
	private void assertAllOrderingValidates(String openTag, String dataEl, String propsEl, String relsEl, String closeTag)
		throws SAXException, IOException
	{
		XMLTestUtil.assertXMLStringValidatesSAX(openTag + dataEl + propsEl + relsEl + closeTag);
		XMLTestUtil.assertXMLStringValidatesSAX(openTag + dataEl + relsEl + propsEl + closeTag);
		XMLTestUtil.assertXMLStringValidatesSAX(openTag + propsEl + dataEl + relsEl + closeTag);
		XMLTestUtil.assertXMLStringValidatesSAX(openTag + propsEl + relsEl + dataEl + closeTag);
		XMLTestUtil.assertXMLStringValidatesSAX(openTag + relsEl + dataEl + propsEl + closeTag);
		XMLTestUtil.assertXMLStringValidatesSAX(openTag + relsEl + propsEl + dataEl + closeTag);
	}
		
	
	/** Ensures the <data>, <properties>, and <relationships> elements may not be duplicated */
	public void testDuplicateElementsNotAllowed()
		throws SAXException, IOException
	{
		// data element
		String inputXML = OPEN_AGENT_INPUT_XML + FEDORA_DC_DATA + FEDORA_DC_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + SERVICE_DESC_DATA + SERVICE_DESC_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + FORMAT_DATA + FORMAT_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + HARVEST_INFO_DATA + HARVEST_INFO_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + EMPTY_DATA + EMPTY_DATA + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);

		// properties
		inputXML = OPEN_AGENT_INPUT_XML + RES_HOST_ID_PROPS + RES_HOST_ID_PROPS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + EMPTY_PROPS + EMPTY_PROPS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + MD_PROPS + MD_PROPS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + MDP_PROPS + MDP_PROPS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + RES_URL_ID_PROPS + RES_URL_ID_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	
		// relationships
		inputXML = OPEN_AGENT_INPUT_XML + MEMBER_OF_REL + MEMBER_OF_REL + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + AGGR_RELS + AGGR_RELS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + MD_RELS + MD_RELS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + MDP_RELS + MDP_RELS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + MEMBER_OF_REL + MEMBER_OF_REL + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}

	/** crude assurance properties cannot be used in the wrong object context */
	public void testPropertiesToObjects()
		throws SAXException, IOException
	{
		// agent object - agent and resource properties (identifier) allowed
		String inputXML = OPEN_AGENT_INPUT_XML + RES_HOST_ID_PROPS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + MD_PROPS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + MDP_PROPS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);

		// aggregator object - no properties allowed
		inputXML = OPEN_AGGR_INPUT_XML + RES_HOST_ID_PROPS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + MD_PROPS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + MDP_PROPS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);

		// metadata object - metadata properties (uniqueID, itemId) allowed
		inputXML = OPEN_MD_INPUT_XML + MD_PROPS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + RES_HOST_ID_PROPS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + MDP_PROPS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);

		// metadataProvider object - metadataProvider properties (setSpec, setName) allowed
		inputXML = OPEN_MDP_INPUT_XML + MDP_PROPS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + RES_HOST_ID_PROPS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + MD_PROPS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		
		// resource object - resource properties (identifier) allowed
		inputXML = OPEN_RESOURCE_INPUT_XML + RES_HOST_ID_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + MD_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + MDP_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
	
	
	/** crude assurance relationships cannot be used in the wrong object context */
	public void testRelationshipsToObjects()
		throws SAXException, IOException
	{
		// agent object - memberOf relationships allowed
		String inputXML = OPEN_AGENT_INPUT_XML + MEMBER_OF_REL + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + AGGR_RELS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + MD_RELS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + MDP_RELS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);

		// aggregator object - aggregator and memberOf relationships allowed
		inputXML = OPEN_AGGR_INPUT_XML + AGGR_RELS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + MEMBER_OF_REL + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + MD_RELS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + MDP_RELS + CLOSE_AGGR_INPUT_XML;

		// metadata object - metadata relationships allowed
		inputXML = OPEN_MD_INPUT_XML + MD_RELS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + AGGR_RELS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + MDP_RELS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + MEMBER_OF_REL + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);

		// metadataProvider object - metadata provider relationships allowed
		inputXML = OPEN_MDP_INPUT_XML + MDP_RELS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + AGGR_RELS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + MD_RELS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + MEMBER_OF_REL + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		
		// resource object - memberOf relationships allowed
		inputXML = OPEN_RESOURCE_INPUT_XML + MEMBER_OF_REL + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + AGGR_RELS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + MD_RELS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + MDP_RELS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
	

	/** crude assurance data elements cannot be used in the wrong object context */
	public void testDataElementsToObjects()
		throws SAXException, IOException
	{
		// agent - fedoraDC datastream allowed
		String inputXML = OPEN_AGENT_INPUT_XML + FEDORA_DC_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + FORMAT_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + SERVICE_DESC_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + HARVEST_INFO_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		
		// aggregator - agentService (serviceDescription) datastream allowed
		inputXML = OPEN_AGGR_INPUT_XML + SERVICE_DESC_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + FEDORA_DC_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + FORMAT_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGGR_INPUT_XML + HARVEST_INFO_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		
		// metadata - format datastream allowed
		inputXML = OPEN_MD_INPUT_XML + FORMAT_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + FEDORA_DC_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + SERVICE_DESC_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + HARVEST_INFO_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		
		// metadataProvider - harvestInfo, agentService (serviceDescription) datastream allowed
		inputXML = OPEN_MDP_INPUT_XML + SERVICE_DESC_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + HARVEST_INFO_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + FEDORA_DC_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + FORMAT_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		
		// resource - no datastream allowed
		inputXML = OPEN_RESOURCE_INPUT_XML + FEDORA_DC_DATA + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + FORMAT_DATA + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + HARVEST_INFO_DATA + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + SERVICE_DESC_DATA + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}

	
	
	/** ensure the setSpec property for MetadataProvider objects rejects invalid
	 * setSpec values (values that don't adhere to OAI-PMH setSpec definition) */
	public void testMetadataProviderSetSpecProperty()
		throws SAXException, IOException
	{
		String inputXML = OPEN_MDP_INPUT_XML + MDP_PROPS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + OPEN_PROPS 
								+	"<setSpec>not_a_valid_setSpec_char:=</setSpec>"
								+ "<setName>the equals char is not valid in a setSpec</setName>"
								+ CLOSE_PROPS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
	

	/** ensure the itemId property for Metadata objects rejects invalid values
	 * (values that are not NSDL OAI identifiers */
	public void testMetadatItemIdProperty()
		throws SAXException, IOException
	{
		String inputXML = OPEN_MD_INPUT_XML + MD_PROPS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + OPEN_PROPS 
								+	"<uniqueID>i.can.be.any.xml.schema.token</uniqueID>"
								+ "<itemId>not:a_valid:nsdl_oai_id</itemId>"
								+ CLOSE_PROPS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
	
	/** ensure the identifier property for Resource objects rejects invalid types
	 * and values.
	 */
	public void testResourceIdentifierProperty()
		throws SAXException, IOException
	{
		String inputXML = OPEN_RESOURCE_INPUT_XML + RES_URL_ID_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + OPEN_PROPS + "<identifier type=\"HOST\">www.stuffonmycat.com</identifier>" + CLOSE_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + OPEN_PROPS + "<identifier type=\"USER\">I.M.A. User</identifier>" + CLOSE_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + OPEN_PROPS + "<identifier type=\"OTHER\">some other kind of resource identifier</identifier>"	+ CLOSE_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_RESOURCE_INPUT_XML + OPEN_PROPS + "<identifier type=\"INVALID\">http://www.stuffonmycat.com</identifier>" + CLOSE_PROPS + CLOSE_RESOURCE_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
	}
	
	private final static String BAD_HANDLE = "notA_Handle!";

	/** crude assurance that specific relationships reject invalid handles */
	public void testRelationshipHandles()
		throws SAXException, IOException
	{
		// memberOf relationship
		String inputXML = OPEN_AGENT_INPUT_XML + OPEN_RELS + "<memberOf>" + BAD_HANDLE + "</memberOf>" + CLOSE_RELS + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
		
		// aggregator relationships
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_RELS + "<aggregatorFor>" + BAD_HANDLE + "</aggregatorFor>" + CLOSE_RELS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_RELS + "<associatedWith>" + BAD_HANDLE + "</associatedWith>" + CLOSE_RELS + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	

		// metadata relationships
		inputXML = OPEN_MD_INPUT_XML + OPEN_RELS + "<metadataFor>" + BAD_HANDLE + "</metadataFor>" + CLOSE_RELS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
		inputXML = OPEN_MD_INPUT_XML + OPEN_RELS + "<metadataProviderBy>" + BAD_HANDLE + "</metadataProviderBy>" + CLOSE_RELS + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
		
		// metadataProvider relationships
		inputXML = OPEN_MDP_INPUT_XML + OPEN_RELS + "<metadataProviderFor>" + BAD_HANDLE + "</metadataProviderFor>" + CLOSE_RELS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
		inputXML = OPEN_MDP_INPUT_XML + OPEN_RELS + "<aggregatedBy>" + BAD_HANDLE + "</aggregatedBy>" + CLOSE_RELS + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);	
	}
	
	/** crude assurance that invalid fedora DC datastream (form/data) is rejected */
	public void testFedoraDCDataElement()
		throws SAXException, IOException
	{
		String inputXML = OPEN_AGENT_INPUT_XML + FEDORA_DC_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + OPEN_DATA + "<DC>"
		+ "<not_oai_dc />" + "</DC>"	+ CLOSE_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_AGENT_INPUT_XML + OPEN_DATA + "<DC>"
		+ "<oai_dc:dc xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:oai_dc=\"http://www.openarchives.org/OAI/2.0/oai_dc/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.openarchives.org/OAI/2.0/oai_dc/ http://www.openarchives.org/OAI/2.0/oai_dc.xsd\">"
		+ "<dc:notAnElement>www.mathdl.org Agent</dc:notAnElement>"
		+ "</oai_dc:dc>" + "</DC>"	+ CLOSE_DATA + CLOSE_AGENT_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);		
	}
	
	/** crude assurance that invalid (metadata) format datastream (form/data) is rejected */
	public void testFormatDataElement()
		throws SAXException, IOException
	{
		String inputXML = OPEN_MD_INPUT_XML + FORMAT_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + OPEN_DATA + "<format type=\"notAValidType\" />" + CLOSE_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + OPEN_DATA + "<format type=\"nsdl_dc\">" 
		+ "<not_a_meta_element />"
		+ "</format>" + CLOSE_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + OPEN_DATA + "<format type=\"nsdl_dc\">" 
		+ "<meta>"
		+ "<nsdl_dc:nsdl_dc xmlns:nsdl_dc=\"http://ns.nsdl.org/nsdl_dc_v1.02/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" schemaVersion=\"1.02.000\" xsi:schemaLocation=\"http://ns.nsdl.org/nsdl_dc_v1.02/ http://ns.nsdl.org/schemas/nsdl_dc/nsdl_dc_v1.02.xsd\">"
			+ "<not_a_valid_element />"
		+ "</nsdl_dc:nsdl_dc>"
		+ "</meta>"
		+ "</format>" + CLOSE_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + OPEN_DATA + "<format type=\"nsdl_dc\">" 
		+ "<meta>"
		+ "<nsdl_dc:nsdl_dc xmlns:nsdl_dc=\"http://ns.nsdl.org/nsdl_dc_v1.02/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" schemaVersion=\"1.02.000\" xsi:schemaLocation=\"http://ns.nsdl.org/nsdl_dc_v1.02/ http://ns.nsdl.org/schemas/nsdl_dc/nsdl_dc_v1.02.xsd\">"
			+ "<dc:title>sample title</dc:title>"
		+ "</nsdl_dc:nsdl_dc>"
		+ "</meta>"
		+ "<info /><info />"
		+ "</format>" + CLOSE_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MD_INPUT_XML + OPEN_DATA + "<format type=\"nsdl_dc\">" 
		+ "<meta>"
		+ "<nsdl_dc:nsdl_dc xmlns:nsdl_dc=\"http://ns.nsdl.org/nsdl_dc_v1.02/\" xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" schemaVersion=\"1.02.000\" xsi:schemaLocation=\"http://ns.nsdl.org/nsdl_dc_v1.02/ http://ns.nsdl.org/schemas/nsdl_dc/nsdl_dc_v1.02.xsd\">"
			+ "<dc:title>sample title</dc:title>"
		+ "</nsdl_dc:nsdl_dc>"
		+ "</meta>"
		+ "<info>"
			+ "<invalid />"
		+ "</info>"
		+ "</format>" + CLOSE_DATA + CLOSE_MD_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}	
	
	/** dc:title and dc:description are required in serviceDescription */
	public void testTitleAndDescInServiceDescription()
		throws SAXException, IOException
	{
		String inputXML = OPEN_AGGR_INPUT_XML + SERVICE_DESC_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		// missing title 
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:description>description of Aggregation</dc:description>" 
		+ "<dc:identifier>http://somewhere.over.rainbow</dc:identifier>"
		+ "<dc:type>Collection</dc:type>"
		+ SAMPLE_IMAGE
		+ "<contacts>" + SAMPLE_CONTACT	+ "</contacts>"
		+ "</serviceDescription>"	+ CLOSE_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// missing description 
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:title>title of Aggregation</dc:title>"
		+ "<dc:identifier>http://somewhere.over.rainbow</dc:identifier>"
		+ "<dc:type>Collection</dc:type>"
		+ SAMPLE_IMAGE
		+ "<contacts>" + SAMPLE_CONTACT	+ "</contacts>"
		+ "</serviceDescription>"	+ CLOSE_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// missing title and description
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:identifier>http://somewhere.over.rainbow</dc:identifier>"
		+ "<dc:type>Collection</dc:type>"
		+ "<dc:format>who cares?</dc:format>"
		+ SAMPLE_IMAGE
		+ "<contacts>" + SAMPLE_CONTACT	+ "</contacts>"
		+ "</serviceDescription>"	+ CLOSE_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// title, description out of order
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:description>description of Aggregation</dc:description>" 
		+ "<dc:title>title of Aggregation</dc:title>"
		+ "<dc:identifier>http://somewhere.over.rainbow</dc:identifier>"
		+ SAMPLE_IMAGE
		+ "<contacts>" + SAMPLE_CONTACT	+ "</contacts>"
		+ "</serviceDescription>"	+ CLOSE_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// extra dc elements
		inputXML = OPEN_AGGR_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:title>title of Aggregation</dc:title>"
		+ "<dc:description>description of Aggregation</dc:description>" 
		+ "<dc:identifier>http://somewhere.over.rainbow</dc:identifier>"
		+ "<dc:type>Collection</dc:type>"
		+ "<dc:format>who cares?</dc:format>"
		+ SAMPLE_IMAGE
		+ "<contacts>" + SAMPLE_CONTACT	+ "</contacts>"
		+ "</serviceDescription>"	+ CLOSE_DATA + CLOSE_AGGR_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
	
	/** crude assurance that invalid serviceDescription datastream (form/data) is rejected */
	public void testServiceDescriptionDataElement()
		throws SAXException, IOException
	{
		String inputXML = OPEN_MDP_INPUT_XML + SERVICE_DESC_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA + "<serviceDescription>" 
		+ "</serviceDescription>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA + "<serviceDescription>"
		+ "<not_a_dc_element />"
		+ "</serviceDescription>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:description>need 3-4 dc elements, not one</dc:description>" 
		+ "</serviceDescription>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:title>title</dc:title>"
		+ "<dc:description>description</dc:description>" 
		+ "<dc:identifier>http://way.up.high</dc:identifier>"
		+ SAMPLE_IMAGE + SAMPLE_IMAGE
		+ "</serviceDescription>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA + "<serviceDescription xmlns:dc=\"http://purl.org/dc/elements/1.1/\">"
		+ "<dc:title>title</dc:title>"
		+ "<dc:description>description</dc:description>" 
		+ "<dc:identifier>http://way.up.high</dc:identifier>"
		+ SAMPLE_IMAGE 
		+ "<contacts>"
		+ "<not_a_real_contact />"
		+ "</contacts>"
		+ "</serviceDescription>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
	
	/** crude assurance that invalid harvestInfo datastream (form/data) is rejected */
	public void testHarvestInfoElement()
		throws SAXException, IOException
	{
		String inputXML = OPEN_MDP_INPUT_XML + HARVEST_INFO_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringValidatesSAX(inputXML);
		// wrong namespace
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA + "<harvestInfo>"
		+ "</harvestInfo>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// no schema location
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA 
		+ "<harvestInfo xmlns=\"http://ns.nsdl.org/MRingest/harvest_v1.00/\">"
		+ "</harvestInfo>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// wrong child
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA 
		+ "<harvestInfo xmlns=\"http://ns.nsdl.org/MRingest/harvest_v1.00/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://ns.nsdl.org/MRingest/harvest_v1.00/ http://ns.nsdl.org/schemas/ndr/ndr_ingest.xsd\">"
		+ "<invalid/>"
		+ "</harvestInfo>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
		// incomplete descendents
		inputXML = OPEN_MDP_INPUT_XML + OPEN_DATA 
		+ "<harvestInfo xmlns=\"http://ns.nsdl.org/MRingest/harvest_v1.00/\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://ns.nsdl.org/MRingest/harvest_v1.00/ http://ns.nsdl.org/schemas/ndr/ndr_ingest.xsd\">"
		+ "<harvestRequest schemaVersion=\"1.00.000\">"
		+ "<baseURL>http://an.oaiserver.baseurl</baseURL>"
		+ "</harvestRequest>"
		+ "</harvestInfo>" + CLOSE_DATA + CLOSE_MDP_INPUT_XML;
		XMLTestUtil.assertXMLStringDoesNotValidateSAX(inputXML);
	}
}
