/*
 * National Science Digital Library - Core Integration
 * Cornell University Information Science
 * 
 * Created on Oct 4, 2006
 */
package org.nsdl.repository.api.action;

import org.nsdl.repository.api.actionForm.Data;

import org.nsdl.util.date.DateUtility;
import org.nsdl.util.date.DateUtilityException;

import java.io.File;
import java.util.Date;

import servletunit.struts.MockStrutsTestCase;


public abstract class BaseActionMockTestCase extends MockStrutsTestCase
{
	protected static final String MOCK_BASE_URL = "http://strutsMockObjectTest.nsdl.org/ndr/api";
	
	protected static final String INPUT_XML_ARGNAME = "inputXML";

	/** system property that can be set to indicate parent directory of WEB-INF */
	public final static String PROP_TEST_CONFIG_DIR = "test.config.dir";

	/** system property that can be set to indicate absolute location of struts-config.xml for testing */
	public final static String PROP_TEST_STRUTS_CONFIG_XML_FILE = "test.struts-config.xml.file";

	/** system property that can be set to indicate absolute location of web.xml for testing */
	public final static String PROP_TEST_WEB_XML_FILE = "test.web.xml.file";
	

	/** constructor present to allow TestSuite with specificly ordered tests */
	public BaseActionMockTestCase(String testname)
	{
		super(testname);
	}
	
	public void setUp()
		throws Exception
	{
		super.setUp();

		// get the correct context directory
		String configDir = System.getProperty(PROP_TEST_CONFIG_DIR);
		if (configDir == null || configDir.length() < 1)
			configDir = "src" + File.separator + "config";
		setContextDirectory(new File(configDir));
	
		final String webinfDir = configDir + File.separator + "WEB-INF" + File.separator;
		
		// get the correct struts-config.xml
		String strutsConfigFile = System.getProperty(PROP_TEST_STRUTS_CONFIG_XML_FILE);
		if (strutsConfigFile == null || strutsConfigFile.length() < 1)
			strutsConfigFile = webinfDir + "struts-config.xml";
		setConfigFile(strutsConfigFile);		
		
		// get the correct web.xml
		String webXmlFile = System.getProperty(PROP_TEST_WEB_XML_FILE);
		if (webXmlFile == null || webXmlFile.length() < 1)
			webXmlFile = webinfDir + "web.xml";
		setServletConfigFile(webXmlFile);
	}
	
	/** perform a struts action that uses a Data form bean; validate response 
	 * properties common to all such actions. 
	 * @param actionRequestPath - the request's path
	 * @param successForwardStr - the literal used for forward for successful
	 * execution of the action */
	protected Data getDataResponse(String actionRequestPath, String successForwardStr)
	{		
		performAction(actionRequestPath);
		verifyForward(successForwardStr);

		return getBasicDataRespone(actionRequestPath);
	}

	/** perform a struts action and validate response data common to all actions. 
	 * @param actionRequestPath - the request's path
	 * @param successForwardStr - the literal used for forward for successful
	 * execution of the action */
	protected void assertErrorResponse(String actionRequestPath, String[] expectedActionErrs, String[] expectedActionMsgs)
	{		
		performAction(actionRequestPath);
		verifyForward(GenericAction.ERROR_FORWARD);
		verifyActionErrors(expectedActionErrs);		
		verifyActionMessages(expectedActionMsgs);
	}

	// ----------------------------------------------------------- private methods

	/** sets the action's request path and requestURL, then performs the action
	 * @param actionRequestPath - the request's path
	 */
	private void performAction(String actionRequestPath)
	{
		setRequestPathInfo(actionRequestPath);
		request.setRequestURL(MOCK_BASE_URL + actionRequestPath);
		actionPerform();
	}

	private Data getBasicDataRespone(String actionRequestPath)
	{
		Data dataBean = (Data) getActionForm();

		// check for errors
		verifyNoActionErrors();
		verifyNoActionMessages();		
		assertNull("Had error: " + dataBean.getCode(), dataBean.getCode());		

		// check requestUrl string
		String reqUrlStr = dataBean.getRequestURLStr();
		assertNotNull("Null requestURLStr", reqUrlStr);
		assertTrue("returned request string must end with " + actionRequestPath, reqUrlStr.endsWith(actionRequestPath));

		// check responseTime string
		String responseTimeStr = dataBean.getResponseTimeStr();
		assertNotNull("Null responseTimeStr", responseTimeStr);
		try
		{
			Date docDate = DateUtility.getDateFromISO8601Plus(responseTimeStr);
			// TODO: check if date is reasonable value
		}
		catch (DateUtilityException e)
		{
			fail("responseTime is not in correct format: " + e.getMessage());
		}
				
		return dataBean;
	}
	
	
}
