/*
 * National Science Digital Library - Core Integration
 * Cornell University Information Science
 * 
 * Created on Oct 4, 2006
 */
package org.nsdl.repository.api.action;

import org.nsdl.mptstore.rdf.Triple;
import org.nsdl.repository.api.FedoraPlugIn;
import org.nsdl.repository.api.actionForm.Data;
import org.nsdl.repository.api.risearch.MPTTestQueryEngine;
import org.nsdl.repository.api.util.ObjectTripleFetcher;
import org.nsdl.repository.impl.fedora.MockFedoraAccess;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.Properties;

import junit.extensions.TestSetup;
import junit.framework.*;

public class FindActionMockTest extends BaseActionMockTestCase
{
	/** public visibility so other tests can use it */
	public static final String FIND_REQUEST_PATH = ActionPathType.FIND.toString();
	
	// FIXME:  success string should be constant in Action class
	private static final String SUCCESS_STR = GenericAction.SUCCESS_FORWARD;
	
	private static final String TRIPLE_SRC_REPOS_URL = "http://repo1.nsdl.org:8880";

	private static MPTTestQueryEngine mpt = null;
	private static MockFedoraAccess fedora = new MockFedoraAccess();
	
	public static Test suite()
	{
		TestSuite testSuite = new TestSuite(FindActionMockTest.class);
		TestSetup wrapper = new TestSetup(testSuite)
		{
			public void setUp() 
				throws IOException, InterruptedException, ParseException
			{
				// set up mptstore test query engine
				mpt = new MPTTestQueryEngine();
				Properties props = new Properties();
				props.setProperty(MPTTestQueryEngine.PROP_DB_DRIVER_CLASSNAME, "org.h2.Driver");
				props.setProperty(MPTTestQueryEngine.PROP_DB_USER, "test");
				props.setProperty(MPTTestQueryEngine.PROP_DB_PASSWD, "test");
				props.setProperty(MPTTestQueryEngine.PROP_DDL_GENERATOR_CLASSNAME, "org.nsdl.mptstore.impl.h2.H2DDLGenerator");
				props.setProperty(MPTTestQueryEngine.PROP_JDBC_URI, "jdbc:h2:mem:test");
				props.setProperty(MPTTestQueryEngine.PROP_MAP, "tmap");
				props.setProperty(MPTTestQueryEngine.PROP_PREFIX, "t");
				mpt.init(props);
				
				// generate triples from repos big bang objects and load into mptstore
				Iterator<Triple> tripleIt = ObjectTripleFetcher.getTriplesFromRepository(TRIPLE_SRC_REPOS_URL, ObjectTripleFetcher.BIG_BANG_PIDS);
				mpt.populate(tripleIt);
				fedora.setRI(mpt);
			}
		};
		return wrapper;
	}
	
	
	public void setUp()
		throws Exception
	{
		super.setUp();

		// set up mock environment with mptstore RIQueryEngine
		
		// NOTE:  the following calls .init() from the ActionServlet, which looks
		//  for WEB-INF/web.xml and for WEB-INF/classes, So the parent of these must
		// be in the classpath
		
		/* Place this gateway in the servlet context */
		getActionServlet().getServletContext().setAttribute(FedoraPlugIn.FEDORA_ACCESS, fedora);
	}

	/** constructor present to allow TestSuite with specificly ordered tests */
	public FindActionMockTest(String testname)
	{
		super(testname);
	}

	public void testNullInputXML()
	{
		// FIXME: find request should do what with null inputXML?
		prepareFindAction(null);
		// FIXME: which error messages are expected?
		assertErrorResponse(FIND_REQUEST_PATH, new String[0], new String[0]);
	}
	
	public void testMemberOfQuery()
	{
		
    /* Get agent authorization */
//    AgentAuthInfo auth = (AgentAuthInfo) getRequest.getAttribute(AuthGateway.AUTH_INFO_KEY);
		
		// FIXME: should do what?
		String inputXML = null;
		prepareFindAction(inputXML);
		Data dataBean = getDataResponse(FIND_REQUEST_PATH, SUCCESS_STR);

		// TODO: examine dataBean properties for correct data
	}

	// ----------------------------------------------------------- private methods

	private void prepareFindAction(String inputXMLValStr)
	{
		addRequestParameter(INPUT_XML_ARGNAME, inputXMLValStr);
	}
	
}
