/*
 * National Science Digital Library - Core Integration
 * Cornell University Information Science
 * 
 * Created on Oct 10, 2006
 */
package org.nsdl.repository.api.action;

import org.nsdl.repository.api.FedoraPlugIn;
import org.nsdl.repository.impl.fedora.FedoraAccess;
import org.nsdl.util.date.*;
import org.nsdl.util.io.SystemPrintUtil;
import org.nsdl.util.xml.XMLutil;

import org.apache.xerces.parsers.DOMParser;

import org.w3c.dom.*;
import org.xml.sax.*;

//import servletunit.struts.CactusStrutsTestCase;
import org.apache.cactus.ServletTestCase;
import com.meterware.httpunit.WebResponse;


import java.io.*;
import java.util.Date;

import javax.servlet.*;


/** Test fixture for testing that XML produced by jsp is schema valid.
 * @author Naomi Dushay
 */
// FIXME: due to a vain attempt to get cactus working for XML schema validation of jsp output
public class SchemaValidCactusTestCase extends ServletTestCase implements ErrorHandler
//public class SchemaValidCactusTestCase extends CactusStrutsTestCase implements ErrorHandler
{
	/** system property that can be set to indicate parent directory of WEB-INF */
	public final static String PROP_TEST_CONFIG_DIR = BaseActionMockTestCase.PROP_TEST_CONFIG_DIR;

	/** system property that can be set to indicate absolute location of struts-config.xml for testing */
	public final static String PROP_TEST_STRUTS_CONFIG_XML_FILE = BaseActionMockTestCase.PROP_TEST_STRUTS_CONFIG_XML_FILE;

	/** system property that can be set to indicate absolute location of web.xml for testing */
	public final static String PROP_TEST_WEB_XML_FILE = BaseActionMockTestCase.PROP_TEST_WEB_XML_FILE;

	protected static final String API_RESP_NS = "http://ns.nsdl.org/ndr/response_v1.00/";
	
// FIXME: this needs to be a property!
// laptop
//	protected static final String CACTUS_URL_PFX = "http://localhost:8080/ndrTest";
	// desktop
	protected static final String CACTUS_URL_PFX = "http://localhost:8080/ndrTest/ServletRedirector";

	private DOMParser parser = new DOMParser();
	
	/** constructor present to create TestSuite with specificly ordered tests */
	public SchemaValidCactusTestCase(String testname)
	{
		super(testname);
		setXMLParserAttributes();
	}	

	public void setUp()
		throws Exception
	{
		super.setUp();

//	 FIXME: may not be needed - trying to initialize stuff in a vain attempt to get cactus working for XML schema validation of jsp output
		// Initialize the fedora access class
    FedoraAccess repository = (FedoraAccess) request.getSession().getServletContext().getAttribute(FedoraPlugIn.FEDORA_ACCESS);

// cactus should slurp these files from war file being tested
/*	
		config.getServletContext();

		// get the correct context directory
		String configDir = System.getProperty(PROP_TEST_CONFIG_DIR);
		if (configDir == null || configDir.length() < 1)
			configDir = "src" + File.separator + "config";
		setContextDirectory(new File(configDir));
	
		final String webinfDir = configDir + File.separator + "WEB-INF" + File.separator;
		
		// get the correct struts-config.xml
		String strutsConfigFile = System.getProperty(PROP_TEST_STRUTS_CONFIG_XML_FILE);
		if (strutsConfigFile == null || strutsConfigFile.length() < 1)
			strutsConfigFile = webinfDir + "struts-config.xml";
		setConfigFile(strutsConfigFile);		
		
		// get the correct web.xml
		String webXmlFile = System.getProperty(PROP_TEST_WEB_XML_FILE);
		if (webXmlFile == null || webXmlFile.length() < 1)
			webXmlFile = webinfDir + "web.xml";
		setServletConfigFile(webXmlFile);
*/
	}

	
	// ---------------------------------------------------------- web app root url
	private static final String SERVICE_NAME = "NSDL Data Repository API";
	private static final String DOC_URL_STR = "http://ndr.comm.nsdl.org";

	
	public void testRootUrl()
		throws IOException, ServletException
	{
		dispatchRequest("/", this.request, this.response);
	}
	public void endRootUrl(WebResponse webResp)
		throws IOException
	{
		assertBasicServiceHtml(webResp);
	}
	
	// ------------------------------------------------------- unavailable message
	public void testUnavailableMessage()
		throws IOException, ServletException
	{
		dispatchRequest("/unavailable.html", this.request, this.response);
	}
	public void endUnavailableMessage(WebResponse webResp)
		throws IOException
	{
		assertBasicServiceHtml(webResp);
	}
		
	
	private void assertBasicServiceHtml(WebResponse webResp)
		throws IOException
	{
		assertTextHTMLResponse(webResp);
		String text = webResp.getText();
		assertTrue("NDR API web app root url unexpectedly doesn't contain " + SERVICE_NAME, text.toLowerCase().contains(SERVICE_NAME.toLowerCase()));
		assertTrue("NDR API web app root url unexpectedly doesn't contain " + DOC_URL_STR, text.contains(DOC_URL_STR));		
	}
	
	
	//---------------------------------------------------------- protected methods
	

	private static final String CACTUS_URL_PREFIX = "/ServletRedirector";
	
	
	/** gets and parses an XML REST response into a DOM document.  XML Schema 
	 * validation is performed during the parse
	 */
	protected void dispatchRequest(String path, ServletRequest req, ServletResponse resp)
		throws IOException, ServletException
	{
// FIXME: trying stuff in a vain attempt to get cactus working for XML schema validation of jsp output
//String urlStr = CACTUS_URL_PFX + path;
System.out.println("DEBUGGING: About to dispatch request: path = " + path);
//		config.getServletContext().getRequestDispatcher(path).forward(req, resp);
		RequestDispatcher rd = config.getServletContext().getRequestDispatcher(path);
//		RequestDispatcher rd = config.getServletContext().getRequestDispatcher(urlStr);
System.out.println("DEBUGGING: Dispatcher is " + rd.toString());
		rd.forward(req, resp);
	}
	
	/** check that the response has content-type text/xml and response code 200, 
	 * then parse response data into a DOM Document.
	 * @param webResp - populated com.meterware.httpunit.WebResponse 
	 * @return - response data as a DOM document
	 * @throws IOException if trouble getting response data from WebResponse
	 * @throws SAXException if non-fatal problem while parsing response
	 */
	protected Document getRestXMLResponseAsDoc(WebResponse webResp)
		throws IOException, SAXException
	{
		Document resultDoc = getResponseAsDoc(webResp, "text/xml");
//		Document resultDoc = getResponseAsDoc(webResp, "application/xhtml+xml");
		
		// check for correct namespace
		String queryUrlStr = webResp.getURL().toString();
		String errmsg = "Rest response for " + queryUrlStr 
							+ " not in correct namespace of " + API_RESP_NS;
		String ns = resultDoc.getDocumentElement().getNamespaceURI();
		assertEquals(errmsg, ns, API_RESP_NS);		
		return resultDoc;
	}

	/** check that the response has content-type application/xhtml+xml and 
	 * response code 200, then parse response data into a DOM Document.
	 * @param webResp - populated com.meterware.httpunit.WebResponse 
	 * @return - response data as a DOM document
	 * @throws IOException if trouble getting response data from WebResponse
	 * @throws SAXException if non-fatal problem while parsing response
	 */
	protected Document getXHTMLResponseAsDoc(WebResponse webResp)
		throws IOException, SAXException
	{
		return getResponseAsDoc(webResp, "application/xhtml+xml");
	}

	/** check that the response has content-type text/html and response code 200
	 * @param webResp - populated com.meterware.httpunit.WebResponse */
	protected void assertTextHTMLResponse(WebResponse webResp)
	{
		assertContentTypeAndRespCode(webResp, "text/html");
	}

	
	/** check that the response has expected content-type and response code 200, 
	 * then parse response data into a DOM Document.
	 * @param webResp - populated com.meterware.httpunit.WebResponse 
	 * @return - response data as a DOM document
	 * @throws IOException if trouble getting response data from WebResponse
	 * @throws SAXException if non-fatal problem while parsing response
	 */
	protected Document getResponseAsDoc(WebResponse webResp, String expectedContentType)
		throws IOException, SAXException
	{
// FIXME: commented out for now
		//		assertContentTypeAndRespCode(webResp, expectedContentType);
// FIXME: trying stuff in a vain attempt to get cactus working for XML schema validation of jsp output
System.out.println("DEBUGGING: URL was: " + webResp.getURL());
System.out.println("DEBUGGING: response code is: " + webResp.getResponseCode());
System.out.println("DEBUGGING: response content type is: " + webResp.getContentType());
System.out.println("DEBUGGING: content length is: " + webResp.getContentLength());
System.out.println("DEBUGGING: response text is: " + webResp.getText());

		// check for schema validation errors
		InputStream is = webResp.getInputStream();
		parsingErrorCount = 0;
		parsingWarningCount = 0;
		
		Document resultDoc = parseRestResults(is);

System.out.println("DEBUGGING: resultDoc is: " + resultDoc.toString());
		
		String queryUrlStr = webResp.getURL().toString();
		String errmsg = "Parsing or schema validation errors for " + queryUrlStr;
		assertEquals(errmsg, 0, parsingErrorCount);
		
		return resultDoc;
	}


	// TODO: method to get an error response as a document (?)

	protected void assertSingleElementInResponse(Document respDoc, String restElementLocalName)
	{
		// the following does the assertion; we need only ignore the output
		getSingleElementFromResponse(respDoc, restElementLocalName);
	}
	
	protected void assertElementNotInRespDoc(Document respDoc, String restElementLocalName)
	{
		NodeList nodes = respDoc.getElementsByTagNameNS(API_RESP_NS, restElementLocalName);
		assertEquals("unexpectedly found <" + restElementLocalName + "> elements", 0, nodes.getLength());
	}
			
	protected void assertTimestampFormat(Document respDoc, String localname)
	{
		String dateStr = XMLutil.getElementValue(getSingleElementFromResponse(respDoc, localname));
		try
		{
			Date docDate = DateUtility.getDateFromISO8601Plus(dateStr);
			// could check if date was expected value, too
		}
		catch (DateUtilityException e)
		{
			fail(localname + " is not in correct format: " + e.getMessage());
		}
	}
	
	protected void assertReqUrlInResponse(Document respDoc, String expectedUrlPrefix)
	{
		String reqUrl = XMLutil.getElementValue(getSingleElementFromResponse(respDoc, "request"));
		assertTrue("cactus request url didn't start with " + expectedUrlPrefix + " : " + reqUrl, reqUrl.startsWith(expectedUrlPrefix));
	}
	
	//--------------------------------------------------- SAX ErrorHandler methods

	/** used to track number of parsing errors */
	private int parsingErrorCount = 0;

	/** used to track number of parsing warnings */
	private int parsingWarningCount = 0;
	
	/** SAX ErrorHandler - Warning */
	public void warning(SAXParseException ex) throws SAXException
	{
		parsingWarningCount++;
	}

	/** SAX ErrorHandler - Error **/
	public void error(SAXParseException ex) throws SAXException
	{
		parsingErrorCount++;
		String errmsg = XMLutil.niceMessage("XML parsing/validation error", ex);
		SystemPrintUtil.prtlnErr(errmsg);
	}

	/** SAX ErrorHandler - Fatal error - fails if parse isn't clean */
	public void fatalError(SAXParseException ex) throws SAXException
	{
		parsingErrorCount++;
		String errmsg = XMLutil.niceMessage("fatal XML parsing error", ex);
		SystemPrintUtil.prtlnErr(errmsg);
		fail(errmsg);
	}	

	//------------------------------------------------------------ private methods
	
	/** check that the response has expected content-type and response code 200
	 * @param webResp - populated com.meterware.httpunit.WebResponse 
	 * @return - response data as a DOM document
	 */
	private void assertContentTypeAndRespCode(WebResponse webResp, String expectedContentType)
	{
// FIXME:  commented out for now.  Expecting "text/xml" and getting something else - not sure why
//		assertEquals("Content type is wrong", expectedContentType, webResp.getContentType());
		assertEquals("Bad responce code", 200, webResp.getResponseCode());
	}
	
	/** will fail if parse isn't clean */
	private Document parseRestResults(InputStream restStream)
		throws IOException, SAXException
	{
		// parse input file into DOM tree
		parser.parse(new InputSource(restStream));
		return parser.getDocument();
	}
	
	private Element getSingleElementFromResponse(Document respDoc, String localname)
	{
		NodeList nodes = respDoc.getElementsByTagNameNS(API_RESP_NS, localname);
		assertEquals("wrong number of <" + localname + "> elements", 1, nodes.getLength());
		return (Element) nodes.item(0);
	}
	
	/** This method turns features on and assigns this class to be the error 
	 * handler; fails if there's a problem */
	private void setXMLParserAttributes()
	{
		parser.setErrorHandler(this);
		try
		{
			// NOTE: sets DYNAMIC_VALIDATION as well ...
			parser = XMLutil.setDOMParserFeatures(parser, true);
		}
		catch (SAXException e)
		{
			String errmsg = "Unrecognized feature: " + e;
			throw new junit.framework.AssertionFailedError(errmsg);
		}
	}	

}
