/*
 * National Science Digital Library - Core Integration
 * Cornell University Information Science
 * 
 * Created on Oct 11, 2006
 */
package org.nsdl.repository.api.action;

import com.meterware.httpunit.WebResponse;

import org.apache.cactus.WebRequest;
import org.nsdl.repository.BigBangHandles;
import org.xml.sax.SAXException;

import junit.extensions.TestSetup;
import junit.framework.*;

import java.io.*;

import javax.servlet.*;


/** Tests that the XML produced from jsp is schema valid 
 * (for non-error requests)
 * @author Naomi Dushay
 */
public class SchemaValidResponsesTest extends SchemaValidCactusTestCase
{
	private static final String ME = "SchemaValidResponsesTest";
	
 	public static Test suite()
	{
		TestSuite testSuite = new TestSuite("org.nsdl.repository.api.action.SchemaValidResponsesTest");
//		testSuite.addTest(new SchemaValidCactusTestCase("testRootUrl"));
//		testSuite.addTest(new SchemaValidCactusTestCase("testUnavailableMessage"));
//		testSuite.addTest(new SchemaValidResponsesTest("testInfoJSPNoArgs"));
//		testSuite.addTest(new SchemaValidResponsesTest("testDescribeNoArgs"));
		testSuite.addTest(new SchemaValidResponsesTest("testDescribeRootAgent"));
//		testSuite.addTest(new SchemaValidResponsesTest("testDescribeCollOfCollsResourceHandle"));
		
		TestSetup wrapper = new TestSetup(testSuite)
		{
			public void setUp()
				throws Exception
			{
				super.setUp();
				// TODO: code to create mock RI and set up NDR to use it
			}
		};
		return wrapper;
	}
	
	/** constructor present to create TestSuite with specificly ordered tests */
	public SchemaValidResponsesTest(String testname)
	{
		super(testname);
	}	

	//---------------------------------------------------------- info.jsp response
	private static final String INFO_JSP_PATH = "/info.jsp";
	public void testInfoJSPNoArgs()
		throws IOException, ServletException
	{
		dispatchRequest(INFO_JSP_PATH, this.request, this.response);
	}
	public void endInfoJSPNoArgs(WebResponse webResp)
		throws IOException, SAXException
	{
		assertNonErrorResponse(webResp, INFO_JSP_PATH);
		// TODO: need to check more response details - but elsewhere perhaps
	}
	
	
	//--------------------------------------------------------- describe responses
	private static final String DESCRIBE_PATH = ActionPathType.DESCRIBE.toString();
	
	// - - - - - - - - - - - - - - - - - - - - - - - -  describe without arguments 
	public void testDescribeNoArgs()
		throws IOException, ServletException
	{
		dispatchRequest(DESCRIBE_PATH, this.request, this.response);
	}
	public void endDescribeNoArgs(WebResponse webResp)
		throws IOException, SAXException
	{
		assertNonErrorResponse(webResp, DESCRIBE_PATH);
		// TODO: need to check more response details - but elsewhere perhaps
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - -  describe root object

// FIXME: beginDescribeRootAgent is here due to a vain attempt to get cactus working for XML schema validation of jsp output
	public void beginDescribeRootAgent(WebRequest webReq)
	{
//		webReq.setURL("localhost:8080", "/ndrTest", DESCRIBE_PATH, "/" + BigBangHandles.ROOT_AGENT_HANDLE, null);
//		webReq.setURL("localhost:8080", "/ndrTest", DESCRIBE_PATH + "/" + BigBangHandles.ROOT_AGENT_HANDLE, null, null);
	}
	
	public void testDescribeRootAgent()
		throws IOException, ServletException
	{
		dispatchRequest(DESCRIBE_PATH +  "/" + BigBangHandles.ROOT_AGENT_HANDLE, this.request, this.response);
	}
	public void endDescribeRootAgent(WebResponse webResp)
		throws IOException, SAXException
	{
		assertNonErrorResponse(webResp, DESCRIBE_PATH + "/" + BigBangHandles.ROOT_AGENT_HANDLE);
		// TODO: need to check more response details - but elsewhere perhaps
	}

	// - - - - - - - - - - - - - - - - - -  describe coll of colls resource object
	
	public void testDescribeCollOfCollsResourceHandle()
		throws IOException, ServletException
	{
		dispatchRequest(DESCRIBE_PATH + "/" + BigBangHandles.COLL_OF_COLLS_RSRC_HANDLE, this.request, this.response);
	}
	public void endDescribeCollOfCollsResourceHandle(WebResponse webResp)
		throws IOException, SAXException
	{
		assertNonErrorResponse(webResp, DESCRIBE_PATH + "/" + BigBangHandles.COLL_OF_COLLS_RSRC_HANDLE);
		// TODO: need to check more response details - but elsewhere perhaps
	}
	
	
	
	//------------------------------------------------------------ private methods

	private org.w3c.dom.Document assertNonErrorResponse(WebResponse webResp, String reqUrlStr)
		throws IOException, SAXException
	{
		org.w3c.dom.Document respDoc = getRestXMLResponseAsDoc(webResp);
		assertReqUrlInResponse(respDoc, CACTUS_URL_PFX + reqUrlStr);
		assertTimestampFormat(respDoc, "responseTime");
		// TODO: check if response time is pretty close to now (in super class)
//		assertNoErrorCodeInResponse(respDoc);
		return respDoc;
	}

}
