package org.nsdl.repository.handle;

import java.util.Iterator;
import java.util.Properties;

import junit.framework.TestCase;

import org.nsdl.repository.error.RepositoryException;

/**
 * @author Edwin Shin
 */
public class TestHandleManager extends TestCase {
    private static Properties props = new Properties();
    static {
        props.put("handle.generator", "org.nsdl.repository.handle.RandomHandleGenerator");
        props.put("handle.na", "2200");
        props.put("handle.admin.index", "300");
        props.put("handle.admin.group.index", "200");
        props.put("handle.admin.privateKey", "/path/to/svr_1/admpriv.bin");
        props.put("handle.admin.passphrase", "very secret");
    }
    
    private HandleManager hm;
    
    protected void setUp() {
        hm = new HandleManager(props);
    }
    
    protected void tearDown() {
        hm = null;
    }
    
    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestHandleManager.class);
    }

    public void testCreateHandle() throws Exception {
        Handle h = hm.createHandle();
        assertEquals(h.toString(), (hm.getHandle(h.toString())).toString());
        hm.deleteHandle(h.toString());
    }
    
    public void testCreateDuplicateHandle() throws Exception {
        Handle h = hm.createHandle();
        try {
            hm.createHandle(h.toString(), null);
            fail("Should have thrown a RepositoryException");
        } catch (RepositoryException e) {
            assertTrue(e.getMessage().indexOf("Error(101): HANDLE ALREADY EXISTS") != -1);
        } finally {
            hm.deleteHandle(h.toString());
        }
    }
    
    public void testListHandles() {
        HandleManager hm = new HandleManager(props);
        Iterator hit = hm.listHandles();
        while (hit.hasNext()) {
            System.out.println((String)hit.next());
        }
    }
}
